@@ -63,7 +63,6 @@
 #include <kstdaction.h>
 #include <khelpmenu.h>
 #include <kedittoolbar.h>
-#include <kkeydialog.h>
 #include <kstatusbar.h>
 #include <kconfig.h>
 #include <kio/netaccess.h>
@@ -111,8 +110,8 @@ int ddp_sock = -1;              /* Appletalk DDP socket */
 
 KMainWidget::KMainWidget(bool bStartDocked)
     : KGetIface( "KGet-Interface" ),
-      KMainWindow(0, "kget mainwindow"),
-      prefDlg( 0L )
+      KMainWindow(0, "kget mainwindow",0),
+      prefDlg( 0 ), kdock( 0 )
 {
 #ifdef _DEBUG
     sDebugIn << endl;
@@ -125,7 +124,7 @@ KMainWidget::KMainWidget(bool bStartDocked)
         cfg.sync();
     }
 
-    b_online = TRUE;
+    b_connected = TRUE;
     b_viewLogWindow = FALSE;
     b_viewPreferences = FALSE;
 
@@ -169,15 +168,14 @@ KMainWidget::KMainWidget(bool bStartDocked)
 
     // setup socket for checking connection
     if ((_sock = sockets_open()) < 0) {
-        log(i18n("Couldn't create valid socket"), false);
+        log(i18n("Could not create valid socket"), false);
     } else {
         connectionTimer->start(5000);   // 5 second interval for checking connection
     }
 
     checkOnline();
-    if (!b_online) {
-        log(i18n("Starting offline"));
-    }
+    ksettings.b_offline=( !b_connected || ksettings.b_offlineMode );
+
     // Setup animation timer
     animTimer = new QTimer(this);
     animCounter = 0;
@@ -211,6 +209,7 @@ KMainWidget::KMainWidget(bool bStartDocked)
     // Setup special windows
     kdrop = new DropTarget();
     kdock = new DockWidget(this);
+    connect(kdock, SIGNAL(quitSelected()), SLOT(slotQuit()));
 
     // Set geometry
     if (ksettings.mainPosition.x() != -1) {
@@ -235,15 +234,16 @@ KMainWidget::KMainWidget(bool bStartDocked)
     m_paAutoShutdown->setChecked(ksettings.b_autoShutdown);
 
 
-    m_paOfflineMode->setChecked(ksettings.b_offlineMode);
-
-
-    if (ksettings.b_offlineMode)
-        setCaption(i18n("Offline"), false);
-    else {
-        setCaption(QString::null, false);
+    m_paOfflineMode->setChecked(ksettings.b_offline);
+    if (!ksettings.b_offlineMode) 
         m_paOfflineMode->setIconSet(LOAD_ICON("tool_offline_mode_on"));
-    }
+
+    if (ksettings.b_offline) {
+        setCaption(i18n("Offline"), false);
+        log(i18n("Starting offline"));
+    } else
+        setCaption(QString::null, false);
+    
     m_paAutoPaste->setChecked(ksettings.b_autoPaste);
     m_paShowLog->setChecked(b_viewLogWindow);
 
@@ -350,7 +350,7 @@ void KMainWidget::setupGUI()
 #endif
     m_paResume = new KAction(i18n("&Resume"),"tool_resume", 0, this, SLOT(slotResumeCurrent()), coll, "resume");
     m_paPause = new KAction(i18n("&Pause"),"tool_pause", 0, this, SLOT(slotPauseCurrent()), coll, "pause");
-    m_paDelete = new KAction(i18n("&Delete"),"tool_delete", 0, this, SLOT(slotDeleteCurrent()), coll, "delete");
+    m_paDelete = new KAction(i18n("&Delete"),"tool_delete", Qt::Key_Delete, this, SLOT(slotDeleteCurrent()), coll, "delete");
     m_paRestart = new KAction(i18n("Re&start"),"tool_restart", 0, this, SLOT(slotRestartCurrent()), coll, "restart");
 
     m_paQueue = new KRadioAction(i18n("&Queue"),"tool_queue", 0, this, SLOT(slotQueueCurrent()), coll, "queue");
@@ -373,13 +373,15 @@ void KMainWidget::setupGUI()
 
     m_paPreferences    =  KStdAction::preferences(this, SLOT(slotPreferences()), coll);
 
-    KStdAction::keyBindings(this, SLOT(slotConfigureKeys()), coll);
+    KStdAction::keyBindings(guiFactory(), SLOT(configureShortcuts()),
+actionCollection());
     KStdAction::configureToolbars(this, SLOT(slotConfigureToolbars()), coll);
 
     // view actions
     createStandardStatusBarAction();
 
     m_paShowLog      = new KToggleAction(i18n("Show &Log Window"),"tool_logwindow", 0, this, SLOT(slotToggleLogWindow()), coll, "toggle_log");
+    m_paShowLog->setCheckedState(i18n("Hide &Log Window"));
     m_paDropTarget   = new KToggleAction(i18n("Drop &Target"),"tool_drop_target", 0, this, SLOT(slotToggleDropTarget()), coll, "drop_target");
 
     menuHelp = new KHelpMenu(this, KGlobal::instance()->aboutData());
@@ -479,21 +481,6 @@ void KMainWidget::setupWhatsThis()
 }
 
 
-void KMainWidget::slotConfigureKeys()
-{
-#ifdef _DEBUG
-    sDebugIn << endl;
-#endif
-
-    KKeyDialog::configure(actionCollection());
-
-
-#ifdef _DEBUG
-    sDebugOut << endl;
-#endif
-}
-
-
 void KMainWidget::slotConfigureToolbars()
 {
 #ifdef _DEBUG
@@ -708,6 +695,7 @@ void KMainWidget::slotQuit()
 
                 return;
             }
+            break;
         }
     }
 
@@ -858,22 +846,22 @@ void KMainWidget::slotDeleteCurrent()
 }
 
 
-void KMainWidget::pauseAll()
+void KMainWidget::stopAll()
 {
 #ifdef _DEBUG
     sDebugIn << endl;
 #endif
 
-    log(i18n("Pausing all jobs"), false);
+    log(i18n("Stopping all jobs"), false);
 
     TransferIterator it(myTransferList);
     Transfer::TransferStatus Status;
     for (; it.current(); ++it) {
         Status = it.current()->getStatus();
         if (Status == Transfer::ST_TRYING || Status == Transfer::ST_RUNNING)
-            it.current()->slotRequestPause();
+            it.current()->slotStop();
     }
-
+    slotUpdateActions();
 
 #ifdef _DEBUG
     sDebugOut << endl;
@@ -1191,6 +1179,10 @@ void KMainWidget::addTransfers( const KURL::List& src, const QString& destDir )
                         destFile = destURL;
                     }
                 }
+                else 
+                {
+                  destFile = destURL;
+                }
             }
         }
 
@@ -1286,10 +1278,9 @@ void KMainWidget::checkQueue()
     uint numRun = 0;
     int status;
     Transfer *item;
+    TransferIterator it(myTransferList);
 
-    if (!ksettings.b_offlineMode && b_online) {
-
-        TransferIterator it(myTransferList);
+    if (!ksettings.b_offline) {
 
         // count running transfers
         for (; it.current(); ++it) {
@@ -1303,11 +1294,12 @@ void KMainWidget::checkQueue()
         bool isQuequed;
         for (; it.current() && numRun < ksettings.maxSimultaneousConnections; ++it) {
             item = it.current();
-            isRunning = (item->getStatus() == Transfer::ST_RUNNING) || (item->getStatus() == Transfer::ST_TRYING);
+            isRunning = (item->getStatus() == Transfer::ST_RUNNING) || ((item->getStatus() == Transfer::ST_TRYING));
 
-            isQuequed = (item->getMode() == Transfer::MD_QUEUED);
+            isQuequed = (item->getMode() == Transfer::MD_QUEUED || item->getMode() == Transfer::MD_NEW);
 
-            if (!isRunning && isQuequed && !ksettings.b_offlineMode) {
+            if     (!isRunning && isQuequed && !ksettings.b_offline)
+            {
                 log(i18n("Starting another queued job."));
                 item->slotResume();
                 numRun++;
@@ -1318,11 +1310,19 @@ void KMainWidget::checkQueue()
 
         updateStatusBar();
 
-    } else {
-        log(i18n("Cannot continue offline status"));
+//    } else {//TODO this has to be solved different
+//        log(i18n("Cannot continue offline status"));
     }
 
-
+    it.reset();
+    for (; it.current(); ++it)
+    {
+        item = it.current();
+        if (item->getMode() == Transfer::MD_NEW && item->getStatus() == Transfer::ST_STOPPED)
+        {
+            item->checkCache();
+        }
+    }
 #ifdef _DEBUG
     sDebugOut << endl;
 #endif
@@ -1340,14 +1340,15 @@ void KMainWidget::slotAnimTimeout()
 
     animCounter++;
     if (animCounter == myTransferList->getPhasesNum()) {
+       //updateStatusBar();
         animCounter = 0;
     }
     // update status of all items of transferList
     isTransfer = myTransferList->updateStatus(animCounter);
 
-    if (this->isVisible()) {
+    //if (this->isVisible()) {
         updateStatusBar();
-    }
+    //}
 #ifdef _DEBUG
     //sDebugOut << endl;
 #endif
@@ -1419,8 +1420,8 @@ void KMainWidget::slotStatusChanged(Transfer * item, int _operation)
         else
             item->setMode(Transfer::MD_NONE);
 
-        if (myTransferList->isQueueEmpty()) {
-            // no items in the TransferList or we have donwload all items
+        if (!myTransferList->areTransfersQueuedOrScheduled()) {
+            // no items or only delayed and finished items in the TransferList
             if (ksettings.b_autoDisconnect)
                 onlineDisconnect();
 
@@ -1570,7 +1571,7 @@ void KMainWidget::slotOpenIndividual()
 bool KMainWidget::queryClose()
 {
     if( kapp->sessionSaving())
-	return true;
+        return true;
     hide();
     return false;
 }
@@ -1685,24 +1686,24 @@ void KMainWidget::slotToggleOfflineMode()
     sDebugIn "ksettings.b_offlineMode = " << ksettings.b_offlineMode << endl;
 #endif
 
-
     ksettings.b_offlineMode = !ksettings.b_offlineMode;
-    if (ksettings.b_offlineMode) {
+    ksettings.b_offline=(ksettings.b_offlineMode || !b_connected);
+    if (ksettings.b_offline) {
         log(i18n("Offline mode on."));
-        pauseAll();
+        stopAll();
         setCaption(i18n("Offline"), false);
         m_paOfflineMode->setIconSet(LOAD_ICON("tool_offline_mode_off"));
     } else {
         log(i18n("Offline mode off."));
-        setCaption(i18n(""), false);
+        setCaption(QString::null, false);
         m_paOfflineMode->setIconSet(LOAD_ICON("tool_offline_mode_on"));
     }
-    m_paOfflineMode->setChecked(ksettings.b_offlineMode);
+
+    m_paOfflineMode->setChecked(ksettings.b_offline);
 
 
     slotUpdateActions();
     checkQueue();
-
 #ifdef _DEBUG
     sDebugOut << endl;
 #endif
@@ -1935,9 +1936,9 @@ void KMainWidget::slotUpdateActions()
             }
             // enable PAUSE, RESUME and RESTART only when we are online and not in offline mode
 #ifdef _DEBUG
-            sDebug << "-->ONLINE= " << ksettings.b_offlineMode << endl;
+            sDebug << "-->ONLINE= " << ksettings.b_offline << endl;
 #endif
-            if (item == first_item && !ksettings.b_offlineMode) {
+            if (item == first_item && !ksettings.b_offline) {
                 switch (item->getStatus()) {
                 case Transfer::ST_TRYING:
                 case Transfer::ST_RUNNING:
@@ -2045,7 +2046,7 @@ void KMainWidget::updateStatusBar()
     QString tmpstr;
 
     int totalFiles = 0;
-    int totalSize = 0;
+    KIO::filesize_t totalSize = 0;
     int totalSpeed = 0;
     QTime remTime;
 
@@ -2070,7 +2071,13 @@ void KMainWidget::updateStatusBar()
     statusBar()->changeItem(i18n(" Time: %1 ").arg(remTime.toString()), ID_TOTAL_TIME);
     statusBar()->changeItem(i18n(" %1/s ").arg(KIO::convertSize(totalSpeed)), ID_TOTAL_SPEED);
 
-
+    if (kdock) {
+        tmpstr = i18n(" Transfers: %1 ").arg(myTransferList->childCount()) +
+                 i18n(" Files: %1 ").arg(totalFiles)+
+                 i18n(" Size: %1 ").arg(KIO::convertSize(totalSize))+
+                 i18n(" Time: %1 %2/s").arg(remTime.toString()).arg(KIO::convertSize(totalSpeed));
+        kdock->updateToolTip( tmpstr );
+    }
 #ifdef _DEBUG
     //sDebugOut << endl;
 #endif
@@ -2083,12 +2090,16 @@ void KMainWidget::onlineDisconnect()
     sDebugIn << endl;
 #endif
 
-    if (!b_online) {
+    if (!b_connected) {
         return;
     }
 
     if (!ksettings.b_expertMode) {
-        if (KMessageBox::questionYesNo(this, i18n("Do you really want to disconnect?"), i18n("Question")) != KMessageBox::Yes) {
+        if (KMessageBox::questionYesNo(this, i18n("Do you really want to disconnect?"),
+                                       i18n("Question"),
+                                       KStdGuiItem::yes(), KStdGuiItem::no(),
+                                       "kget_AutoOnlineDisconnect")
+            != KMessageBox::Yes) {
             return;
         }
     }
@@ -2106,9 +2117,21 @@ void KMainWidget::slotCheckConnection()
 #ifdef _DEBUG
     //sDebugIn << endl;
 #endif
-
+    bool old = b_connected;
     checkOnline();
-
+    if (b_connected != old) {
+        if (b_connected) {
+            log(i18n("We are online."));
+            setCaption(QString::null, false);
+            ksettings.b_offline=ksettings.b_offlineMode;
+            checkQueue();
+        } else {
+            log(i18n("We are offline."));
+            setCaption(i18n("Offline"), false);
+            ksettings.b_offline=true;
+            stopAll();
+        }
+    }
 #ifdef _DEBUG
     //sDebugOut << endl;
 #endif
@@ -2121,8 +2144,6 @@ void KMainWidget::checkOnline()
     //sDebugIn << endl;
 #endif
 
-    bool old = b_online;
-
     struct ifreq ifr;
 
     memset(&ifr, 0, sizeof(ifreq));
@@ -2136,40 +2157,28 @@ void KMainWidget::checkOnline()
         // get the flags for particular device
         if (ioctl(_sock, SIOCGIFFLAGS, &ifr) < 0) {
             flag = true;
-            b_online = false;
+            b_connected = false;
         } else if (ifr.ifr_flags == 0) {
 #ifdef _DEBUG
             sDebug << "Can't get flags from interface " << ifr.ifr_name << endl;
 #endif
-            b_online = false;
+            b_connected = false;
         } else if (ifr.ifr_flags & IFF_UP) {    // if (ifr.ifr_flags & IFF_RUNNING)
-            b_online = true;
+            b_connected = true;
         } else {
-            b_online = false;
+            b_connected = false;
         }
     } else {
-        b_online = true;        // PERMANENT connection
+        b_connected = true;        // PERMANENT connection
     }
 
-    m_paOfflineMode->setEnabled(b_online);
+    m_paOfflineMode->setEnabled(b_connected);
 
-    if (b_online != old) {
-        if (flag) {             // so that we write this only once when connection is changed
+    if (flag) {
 #ifdef _DEBUG
             sDebug << "Unknown interface " << ifr.ifr_name << endl;
 #endif
         }
-
-        if (b_online) {
-            log(i18n("We are online!"));
-            checkQueue();
-        } else {
-            log(i18n("We are offline!"));
-            pauseAll();
-        }
-    }
-
-
 #ifdef _DEBUG
     //sDebugOut << endl;
 #endif
@@ -2275,9 +2284,14 @@ void KMainWidget::customEvent(QCustomEvent * _e)
         e->getItem()->slotTotalSize(e->getData());
         break;
 
-    case Slave::SLV_ABORTED:
-        e->getItem()->slotExecAbort(e->getMsg());
+    case Slave::SLV_ERROR:
+        e->getItem()->slotExecError();
         break;
+
+    case Slave::SLV_BROKEN:
+        e->getItem()->slotExecBroken();
+        break;
+
     case Slave::SLV_REMOVED:
         e->getItem()->slotExecRemove();
         break;
@@ -2285,6 +2299,10 @@ void KMainWidget::customEvent(QCustomEvent * _e)
     case Slave::SLV_INFO:
         e->getItem()->logMessage(e->getMsg());
         break;
+        
+        case Slave::SLV_NOTINCACHE:
+        e->getItem()->NotInCache();
+        break;
     default:
 #ifdef _DEBUG
         sDebug << "Unknown Result..die" << result << endl;
