@@ -2,9 +2,8 @@
 *                                transfer.cpp
 *                             -------------------
 *
-*    Revision     : $Id: transfer.cpp,v 1.57.2.1 2004/08/09 18:19:10 tokoe Exp $
 *    begin        : Tue Jan 29 2002
-*    copyright    : (C) 2002 by Patrick Charbonnier
+*    copyright    : (C) 2002, 2003, 2004, 2005 by Patrick Charbonnier
 *                 : Based On Caitoo v.0.7.3 (c) 1998 - 2000, Matej Koss
 *    email        : pch@freeshell.org
 *
@@ -117,12 +116,6 @@ Transfer::init(const uint _id)
 
     status = ST_STOPPED;
 
-    if (ksettings.b_addQueued)
-        mode = MD_QUEUED;
-    else
-        mode = MD_DELAYED;
-
-
 
     connect(this, SIGNAL(statusChanged(Transfer *, int)), kmain, SLOT(slotStatusChanged(Transfer *, int)));
     connect(this, SIGNAL(statusChanged(Transfer *, int)), this, SLOT(slotUpdateActions()));
@@ -154,6 +147,8 @@ Transfer::init(const uint _id)
 
     // setup individual transfer dialog
 
+    mode = MD_NEW;
+
     sDebugOut << endl;
 }
 
@@ -182,9 +177,9 @@ void Transfer::synchronousAbort()
 
 void Transfer::slotUpdateActions()
 {
-    sDebugIn << "the item Status is =" << status << "offline=" << ksettings.b_offlineMode << endl;
-    //if we are in offlinemode just disable all actions  and return
-    if (ksettings.b_offlineMode) {
+    sDebugIn << "the item Status is =" << status << "offline=" << ksettings.b_offline << endl;
+     //if we are offline just disable Resume and Pause and return
+    if (ksettings.b_offline) {
         m_paResume->setEnabled(false);
         m_paPause->setEnabled(false);
         m_paRestart->setEnabled(false);
@@ -196,7 +191,8 @@ void Transfer::slotUpdateActions()
     UpdateRetry();
 
     switch (status) {
-    case ST_TRYING:
+
+    case ST_TRYING://fall-through
     case ST_RUNNING:
         m_paResume->setEnabled(false);
         m_paPause->setEnabled(true);
@@ -230,6 +226,7 @@ void Transfer::slotUpdateActions()
     case MD_DELAYED:
         m_paDelay->setChecked(true);
         break;
+    case MD_NEW: //fall through
     case MD_NONE:
         m_paQueue->setChecked(false);
         m_paTimer->setChecked(false);
@@ -273,8 +270,8 @@ void Transfer::updateAll()
 
     updateStatus(status);       // first phase of animation
 
-    logMessage(i18n("Copy file from: %1").arg(src.url()));
-    logMessage(i18n("To: %1").arg(dest.url()));
+    logMessage(i18n("Copy file from: %1").arg(src.prettyURL()));
+    logMessage(i18n("To: %1").arg(dest.prettyURL()));
 
     // source
     setText(view->lv_url, src.prettyURL());
@@ -310,7 +307,6 @@ bool Transfer::updateStatus(int counter)
 
     QPixmap *pix = 0L;
     bool isTransfer = false;
-    static TransferStatus prevStatus;
 
     view->setUpdatesEnabled(false);
 
@@ -318,6 +314,8 @@ bool Transfer::updateStatus(int counter)
         {
         case ST_RUNNING:
             pix = view->animConn.at(counter);
+            isTransfer = true;
+            break;
         case ST_TRYING:
             pix = view->animTry.at(counter);
             isTransfer = true;
@@ -337,7 +335,7 @@ bool Transfer::updateStatus(int counter)
     setPixmap(view->lv_pixmap, *pix);
     view->setUpdatesEnabled(true);
 
-    if(prevStatus!=status || status==ST_RUNNING || status==ST_TRYING)
+    if(prevStatus!=status || prevMode != mode || status==ST_RUNNING || status==ST_TRYING)
         {
         QRect rect = view->header()->sectionRect(view->lv_pixmap);
 
@@ -349,6 +347,7 @@ bool Transfer::updateStatus(int counter)
         view->QScrollView::updateContents(x,y,w,h);
 
         prevStatus = status;
+       prevMode = mode;
     }
 
     return isTransfer;
@@ -374,12 +373,12 @@ void Transfer::slotResume()
 
     retryCount++;
     if (retryCount > ksettings.reconnectRetries)
-        ksettings.reconnectRetries = retryCount;
+        retryCount = 1;
     UpdateRetry();
     assert(status == ST_STOPPED);
 
-    sDebug << "src: " << src.url() << endl;
-    sDebug << "dest " << dest.url() << endl;
+    sDebug << "src: " << src.prettyURL() << endl;
+    sDebug << "dest " << dest.prettyURL() << endl;
 
     m_paResume->setEnabled(false);
 
@@ -394,6 +393,26 @@ void Transfer::slotResume()
 }
 
 
+ void Transfer::slotStop()
+{
+    sDebugIn << endl;
+
+    logMessage(i18n("Stopping"));
+
+    assert(status <= ST_RUNNING && ksettings.b_offline);
+
+    m_pSlave->Op(Slave::KILL); // KILL doesn't post a Message
+    sDebug << "Killing Slave" << endl;
+
+    slotSpeed(0);
+    mode = MD_QUEUED;
+    status=ST_STOPPED;
+    m_paQueue->setChecked(true);
+
+    slotUpdateActions();
+
+    sDebugOut << endl;
+}
 
 
 void Transfer::slotRequestPause()
@@ -434,14 +453,16 @@ void Transfer::slotRequestRemove()
     m_paDelete->setEnabled(false);
     m_paPause->setEnabled(false);
     if(dlgIndividual)
-		dlgIndividual->close();
+                dlgIndividual->close();
+
 
     if ( status != ST_FINISHED )
     {
-        // delete the partly downloaded file, if any
         KURL file = dest;
+        // delete the partly downloaded file, if any
         file.setFileName( dest.fileName() + ".part" ); // ### get it from the job?
-        if ( KIO::NetAccess::exists( file ) ) // don't pollute user with warnings
+  
+        if ( KIO::NetAccess::exists( file, false, view ) ) // don't pollute user with warnings
         {
             SafeDelete::deleteFile( file ); // ### messagebox on failure?
         }
@@ -550,7 +571,7 @@ void Transfer::slotRenaming(KIO::Job *, const KURL &, const KURL & to)
 
         dest = to;
 
-        logMessage(i18n("Renaming to %1").arg(dest.url().ascii()));
+        logMessage(i18n("Renaming to %1").arg(dest.prettyURL().ascii()));
 
           // destination
           setText (view->lv_filename, dest.fileName ());
@@ -608,7 +629,7 @@ void Transfer::slotTotalSize(KIO::filesize_t bytes)
     if (totalSize == 0) {
         totalSize = bytes;
         if (totalSize != 0) {
-            logMessage(i18n("Total size is %1 bytes").arg((double)totalSize,0,'f'));
+            logMessage(i18n("Total size is %1 bytes").arg((double)totalSize,0,'f',0));
             setText(view->lv_total, KIO::convertSize(totalSize));
 			if(dlgIndividual)
 				{
@@ -746,7 +767,7 @@ bool Transfer::read(KSimpleConfig * config, int id)
     }
 
     if (!src.isValid() && !ksettings.b_expertMode) {
-        KMessageBox::error(kmain, i18n("Malformed URL:\n") + src.url(), i18n("Error"));
+        KMessageBox::error(kmain, i18n("Malformed URL:\n") + src.prettyURL(), i18n("Error"));
         return false;
     }
 
@@ -853,6 +874,25 @@ void Transfer::slotExecResume()
 void Transfer::slotExecConnected()
 {
     sDebugIn << endl;
+    if (mode == MD_NEW)
+    {
+        if (ksettings.b_offline)// when we're offline and arrive here, then the file is in cache
+            return;             // Slave::slotResult will be called immediately, so we do nothing here
+        status = ST_STOPPED;
+        m_pSlave->Op(Slave::KILL);
+        if (ksettings.b_addQueued)
+        {
+            mode = MD_QUEUED;
+            emit statusChanged(this, OP_QUEUED);
+        }
+        else
+        {
+            mode = MD_DELAYED;
+            emit statusChanged(this, OP_DELAYED);
+        }
+        return;
+    }
+    
     status = ST_RUNNING;
     emit statusChanged(this, OP_CONNECTED);
     sDebugOut << endl;
@@ -936,13 +976,35 @@ void Transfer::maybeShow()
 
 bool Transfer::retryOnError()
 {
-    return (ksettings.b_reconnectOnError && (retryCount <= ksettings.reconnectRetries));
+    return (ksettings.b_reconnectOnError && (retryCount < ksettings.reconnectRetries));
 }
 
 bool Transfer::retryOnBroken()
 {
-    return (ksettings.b_reconnectOnBroken && (retryCount <= ksettings.reconnectRetries));
+    return (ksettings.b_reconnectOnBroken && (retryCount < ksettings.reconnectRetries));
 }
 
+void Transfer::checkCache()
+{
+    assert (mode == MD_NEW);
+
+    if (src.protocol()=="http")
+    {
+        status = ST_TRYING;
+        m_pSlave->Op(Slave::RETR_CACHE);
+    }
+    else
+        NotInCache();
+}
+
+void Transfer::NotInCache()
+{
+    logMessage(i18n("checking if file is in cache...no"));
+    if (ksettings.b_addQueued)
+        mode = MD_QUEUED;
+    else
+        mode = MD_DELAYED;
+    status = ST_STOPPED;
+}
 #include "transfer.moc"
 
