@@ -2,7 +2,7 @@
 *                                transfer.h
 *                             -------------------
 *
-*    Revision     : $Id: transfer.h,v 1.26 2004/04/28 19:23:13 mcamen Exp $
+*    Revision     : $Id: transfer.h,v 1.29 2005/02/10 21:42:02 ratz Exp $
 *    begin        : Tue Jan 29 2002
 *    copyright    : (C) 2002 by Patrick Charbonnier
 *                 : Based On Caitoo v.0.7.3 (c) 1998 - 2000, Matej Koss
@@ -54,7 +54,7 @@ class Transfer:public QObject, public KListViewItem
 {
     Q_OBJECT
 public:
-    enum TransferMode { MD_QUEUED, MD_DELAYED, MD_SCHEDULED, MD_NONE };
+    enum TransferMode { MD_QUEUED, MD_DELAYED, MD_SCHEDULED, MD_NONE, MD_NEW };
 
     enum TransferStatus { ST_TRYING, ST_RUNNING, ST_STOPPED, ST_FINISHED };
 
@@ -160,6 +160,7 @@ public:
     void slotExecBroken();
     void slotCanResume(bool _bCanResume);
     void slotSpeed(unsigned long);
+    void checkCache();
 
     bool isVisible() const;
     void maybeShow();
@@ -174,9 +175,8 @@ public slots:
     void slotRequestRemove();
     void slotRequestSchedule();
     void slotRequestDelay();
-
+    void NotInCache();
     void slotRequestRestart();
-
     void slotUpdateActions();
 
     void slotQueue();
@@ -186,6 +186,7 @@ public slots:
     void slotProcessedSize(KIO::filesize_t);
 
     void slotStartTime(const QDateTime &);
+    void slotStop(); // stop all transfers when going offline
     
 signals:
     void statusChanged(Transfer *, int _operation);
@@ -220,6 +221,9 @@ private:
     TransferStatus status;
     TransferMode mode;
 
+   TransferStatus prevStatus;
+   TransferMode prevMode;
+
     // how many times have we retried already
     unsigned int retryCount;
 
