@@ -2,7 +2,7 @@
     kopeteaccountmanager.cpp - Kopete Account Manager
 
     Copyright (c) 2002-2003 by Martijn Klingens      <klingens@kde.org>
-    Copyright (c) 2003      by Olivier Goffart       <ogoffart@tiscalinet.be>
+    Copyright (c) 2003-2004 by Olivier Goffart       <ogoffart@tiscalinet.be>
 
     Kopete    (c) 2002-2004 by the Kopete developers <kopete-devel@kde.org>
 
@@ -32,98 +32,114 @@
 #include "kopeteprotocol.h"
 #include "kopetecontact.h"
 #include "kopetepluginmanager.h"
+#include "kopeteonlinestatus.h"
 
-class KopeteAccountPtrList : public QPtrList<KopeteAccount>
-{
-	protected:
-		int compareItems( KopeteAccountPtrList::Item a, KopeteAccountPtrList::Item b )
-		{
-			uint priority1 = static_cast<KopeteAccount*>(a)->priority();
-			uint priority2 = static_cast<KopeteAccount*>(b)->priority();
+namespace Kopete {
 
-			if( priority1 == priority2 )
-				return 0;
-			else if( priority1 > priority2 )
-				return 1;
-			else
-				return -1;
-		}
-};
-
-class KopeteAccountManagerPrivate
+class AccountManager::Private
 {
 public:
-	static KopeteAccountManager *s_manager;
 
-	KopeteAccountPtrList accounts;
+	class AccountPtrList : public QPtrList<Account>
+	{
+		protected:
+			int compareItems( AccountPtrList::Item a, AccountPtrList::Item b )
+			{
+				uint priority1 = static_cast<Account*>(a)->priority();
+				uint priority2 = static_cast<Account*>(b)->priority();
+	
+				if( a==b ) //two account are equal only if they are equal :-)
+					return 0;  // remember than an account can be only once on the list, but two account may have the same priority when loading
+				else if( priority1 > priority2 )
+					return 1;
+				else
+					return -1;
+			}
+	} accounts;
+
 };
 
-KopeteAccountManager * KopeteAccountManagerPrivate::s_manager = 0L;
+AccountManager * AccountManager::s_self = 0L;
 
-KopeteAccountManager * KopeteAccountManager::manager()
+AccountManager * AccountManager::self()
 {
-	if ( !KopeteAccountManagerPrivate::s_manager )
-		KopeteAccountManagerPrivate::s_manager = new KopeteAccountManager;
+	if ( !s_self )
+		s_self = new AccountManager;
 
-	return KopeteAccountManagerPrivate::s_manager;
+	return s_self;
 }
 
-KopeteAccountManager::KopeteAccountManager()
+
+AccountManager::AccountManager()
 : QObject( qApp, "KopeteAccountManager" )
 {
-	d = new KopeteAccountManagerPrivate;
+	d = new Private;
 }
 
-KopeteAccountManager::~KopeteAccountManager()
+
+AccountManager::~AccountManager()
 {
-	KopeteAccountManagerPrivate::s_manager = 0L;
+	s_self = 0L;
 
 	delete d;
 }
 
-void KopeteAccountManager::connectAll()
+void AccountManager::connectAll()
 {
-	for ( QPtrListIterator<KopeteAccount> it( d->accounts ); it.current(); ++it )
-		it.current()->connect();
+	for ( QPtrListIterator<Account> it( d->accounts ); it.current(); ++it )
+		if(!it.current()->excludeConnect())
+			it.current()->connect();
 }
 
-void KopeteAccountManager::disconnectAll()
+void AccountManager::setAvailableAll()
 {
-	for ( QPtrListIterator<KopeteAccount> it( d->accounts ); it.current(); ++it )
+	Away::setGlobalAway( false );
+	bool anyConnected = false;
+	for ( QPtrListIterator<Account> it( d->accounts ); it.current(); ++it )
+	{
+		anyConnected |= it.current()->isConnected();
+	}
+	
+	for ( QPtrListIterator<Account> it( d->accounts ); it.current(); ++it )
+	{
+		if ( anyConnected )
+		{
+			if ( it.current()->isConnected() && it.current()->isAway() )
+				it.current()->setAway( false );
+		}
+		else 
+			if(!it.current()->excludeConnect())
+				it.current()->connect();
+	}
+}
+
+void AccountManager::disconnectAll()
+{
+	for ( QPtrListIterator<Account> it( d->accounts ); it.current(); ++it )
 		it.current()->disconnect();
 }
 
-void KopeteAccountManager::setAwayAll( const QString &awayReason )
+void AccountManager::setAwayAll( const QString &awayReason )
 {
-	KopeteAway::setGlobalAway( true );
+	Away::setGlobalAway( true );
 
-	for ( QPtrListIterator<KopeteAccount> it( d->accounts ); it.current(); ++it )
+	for ( QPtrListIterator<Account> it( d->accounts ); it.current(); ++it )
 	{
 		// FIXME: ICQ's invisible online should be set to invisible away
-		KopeteContact *self = it.current()->myself();
-		bool isInvisible = self && self->onlineStatus().status() == KopeteOnlineStatus::Invisible;
+		Contact *self = it.current()->myself();
+		bool isInvisible = self && self->onlineStatus().status() == OnlineStatus::Invisible;
 		if ( it.current()->isConnected() && !isInvisible )
-			it.current()->setAway( true, awayReason.isNull() ? KopeteAway::message() : awayReason );
+			it.current()->setAway( true, awayReason );
 	}
 }
 
-void KopeteAccountManager::setAvailableAll()
-{
-	KopeteAway::setGlobalAway( false );
-
-	for ( QPtrListIterator<KopeteAccount> it( d->accounts ); it.current(); ++it )
-	{
-		if ( it.current()->isConnected() && it.current()->isAway() )
-			it.current()->setAway( false );
-	}
-}
-
-QColor KopeteAccountManager::guessColor( KopeteProtocol *protocol )
+QColor AccountManager::guessColor( Protocol *protocol ) const
 {
 	// FIXME: Use a different algoritm. It should check if the color is really not
 	//        used - Olivier
 	int protocolCount = 0;
-	for ( QPtrListIterator<KopeteAccount> it( d->accounts ); it.current(); ++it )
+
+	for ( QPtrListIterator<Account> it( d->accounts ); it.current(); ++it )
 	{
 		if ( it.current()->protocol()->pluginId() == protocol->pluginId() )
 			protocolCount++;
@@ -159,32 +175,58 @@ QColor KopeteAccountManager::guessColor( KopeteProtocol *protocol )
 	return color;
 }
 
-bool KopeteAccountManager::registerAccount( KopeteAccount *account )
+Account* AccountManager::registerAccount( Account *account )
 {
-
-	if ( !account || account->accountId().isNull() )
-		return false;
-
-	// If this account already exists, do nothing
-	for ( QPtrListIterator<KopeteAccount> it( d->accounts ); it.current(); ++it )
+	if( !account || d->accounts.contains( account ) )
+		return account;
+		
+	if( account->accountId().isEmpty() )
 	{
-		if ( ( account->protocol() == it.current()->protocol() ) && ( account->accountId() == it.current()->accountId() ) )
-			return false;
+		account->deleteLater();
+		return 0L;
 	}
 
-	d->accounts.append( account );
-	return true;
+	// If this account already exists, do nothing
+	for ( QPtrListIterator<Account> it( d->accounts ); it.current(); ++it )
+	{
+		if ( ( account->protocol() == it.current()->protocol() ) && ( account->accountId() == it.current()->accountId() ) )
+		{
+			account->deleteLater();
+			return 0L;
+		}
+	}
+
+	d->accounts.append( account );	
+	d->accounts.sort();
+	
+	// Connect to the account's status changed signal
+	connect(account->myself(), SIGNAL(onlineStatusChanged(Kopete::Contact *,
+			const Kopete::OnlineStatus &, const Kopete::OnlineStatus &)),
+		this, SLOT(slotAccountOnlineStatusChanged(Kopete::Contact *,
+			const Kopete::OnlineStatus &, const Kopete::OnlineStatus &)));
+
+	connect(account, SIGNAL(accountDestroyed(const Kopete::Account *)) , this, SLOT( unregisterAccount(const Kopete::Account *) ));
+
+	emit accountRegistered( account );
+	return account;
 }
 
-const QPtrList<KopeteAccount>& KopeteAccountManager::accounts() const
+void AccountManager::unregisterAccount( const Account *account )
+{
+	kdDebug( 14010 ) << k_funcinfo << "Unregistering account " << account->accountId() << endl;
+	d->accounts.remove( account );
+	emit accountUnregistered( account );
+}
+
+const QPtrList<Account>& AccountManager::accounts() const
 {
 	return d->accounts;
 }
 
-QDict<KopeteAccount> KopeteAccountManager::accounts( const KopeteProtocol *protocol )
+QDict<Account> AccountManager::accounts( const Protocol *protocol ) const
 {
-	QDict<KopeteAccount> dict;
-	for ( QPtrListIterator<KopeteAccount> it( d->accounts ); it.current(); ++it )
+	QDict<Account> dict;
+	for ( QPtrListIterator<Account> it( d->accounts ); it.current(); ++it )
 	{
 		if ( it.current()->protocol() == protocol && !it.current()->accountId().isNull() )
 			dict.insert( it.current()->accountId(), it.current() );
@@ -193,9 +235,9 @@ QDict<KopeteAccount> KopeteAccountManager::accounts( const KopeteProtocol *proto
 	return dict;
 }
 
-KopeteAccount * KopeteAccountManager::findAccount( const QString &protocolId, const QString &accountId )
+Account * AccountManager::findAccount( const QString &protocolId, const QString &accountId )
 {
-	for ( QPtrListIterator<KopeteAccount> it( d->accounts ); it.current(); ++it )
+	for ( QPtrListIterator<Account> it( d->accounts ); it.current(); ++it )
 	{
 		if ( it.current()->protocol()->pluginId() == protocolId && it.current()->accountId() == accountId )
 			return it.current();
@@ -203,58 +245,56 @@ KopeteAccount * KopeteAccountManager::findAccount( const QString &protocolId, co
 	return 0L;
 }
 
-void KopeteAccountManager::removeAccount( KopeteAccount *account )
+void AccountManager::removeAccount( Account *account )
 {
-	kdDebug( 14010 ) << k_funcinfo << "Removing account '" <<
-		account->accountId() << "' and cleaning up config" << endl;
+	if(!account->removeAccount())
+		return;
 
-	KopeteProtocol *protocol = account->protocol();
+	Protocol *protocol = account->protocol();
 
-	KConfig *config = KGlobal::config();
-	QString groupName = account->configGroup();
+
+	KConfigGroup *configgroup = account->configGroup();
 
 	// Clean up the account list
 	d->accounts.remove( account );
 
+	// Clean up configuration
+	configgroup->deleteGroup();
+	configgroup->sync();
+
 	delete account;
 
-	// Clean up configuration
-	config->deleteGroup( groupName );
-	config->sync();
-
-	if ( KopeteAccountManager::manager()->accounts( protocol ).isEmpty() )
+	if ( accounts( protocol ).isEmpty() )
 	{
 		// FIXME: pluginId() should return the internal name and not the class name, so
 		//        we can get rid of this hack - Olivier/Martijn
 		QString protocolName = protocol->pluginId().remove( QString::fromLatin1( "Protocol" ) ).lower();
 
-		KopetePluginManager::self()->setPluginEnabled( protocolName, false );
-		KopetePluginManager::self()->unloadPlugin( protocolName );
+		PluginManager::self()->setPluginEnabled( protocolName, false );
+		PluginManager::self()->unloadPlugin( protocolName );
 	}
 }
 
-void KopeteAccountManager::unregisterAccount( KopeteAccount *account )
-{
-	/*kdDebug( 14010 ) << k_funcinfo << "Unregistering account " <<
-		account->accountId() << endl;*/
-	d->accounts.remove( account );
-	emit accountUnregistered( account );
-}
-
-void KopeteAccountManager::save()
+void AccountManager::save()
 {
 	//kdDebug( 14010 ) << k_funcinfo << endl;
 	d->accounts.sort();
-	for ( QPtrListIterator<KopeteAccount> it( d->accounts ); it.current(); ++it )
-		it.current()->writeConfig( it.current()->configGroup() );
+	
+	for ( QPtrListIterator<Account> it( d->accounts ); it.current(); ++it )
+	{
+		KConfigBase *config = it.current()->configGroup();
+	
+		config->writeEntry( "Protocol", it.current()->protocol()->pluginId() );
+		config->writeEntry( "AccountId", it.current()->accountId() );
+	}
 
 	KGlobal::config()->sync();
 }
 
-void KopeteAccountManager::load()
+void AccountManager::load()
 {
-	connect( KopetePluginManager::self(), SIGNAL( pluginLoaded( KopetePlugin * ) ),
-		this, SLOT( slotPluginLoaded( KopetePlugin * ) ) );
+	connect( PluginManager::self(), SIGNAL( pluginLoaded( Kopete::Plugin * ) ),
+		this, SLOT( slotPluginLoaded( Kopete::Plugin * ) ) );
 
 	// Iterate over all groups that start with "Account_" as those are accounts
 	// and load the required protocols if the account is enabled.
@@ -271,13 +311,13 @@ void KopeteAccountManager::load()
 			protocol = QString::fromLatin1( "kopete_" ) + protocol.lower().remove( QString::fromLatin1( "protocol" ) );
 
 		if ( config->readBoolEntry( "Enabled", true ) )
-			KopetePluginManager::self()->loadPlugin( protocol, KopetePluginManager::LoadAsync );
+			PluginManager::self()->loadPlugin( protocol, PluginManager::LoadAsync );
 	}
 }
 
-void KopeteAccountManager::slotPluginLoaded( KopetePlugin *plugin )
+void AccountManager::slotPluginLoaded( Plugin *plugin )
 {
-	KopeteProtocol* protocol = dynamic_cast<KopeteProtocol*>( plugin );
+	Protocol* protocol = dynamic_cast<Protocol*>( plugin );
 	if ( !protocol )
 		return;
 
@@ -307,44 +347,21 @@ void KopeteAccountManager::slotPluginLoaded( KopetePlugin *plugin )
 		kdDebug( 14010 ) << k_funcinfo <<
 			"Creating account for '" << accountId << "'" << endl;
 		
-		KopeteAccount *account = 0L;
-		account = protocol->createNewAccount( accountId );
+		Account *account = 0L;
+		account = registerAccount( protocol->createNewAccount( accountId ) );
 		if ( !account )
 		{
 			kdWarning( 14010 ) << k_funcinfo <<
 				"Failed to create account for '" << accountId << "'" << endl;
 			continue;
 		}
-		account->readConfig( *it );
 	}
 }
 
-void KopeteAccountManager::autoConnect()
+void AccountManager::slotAccountOnlineStatusChanged(Contact *c,
+	const OnlineStatus &oldStatus, const OnlineStatus &newStatus)
 {
-	for ( QPtrListIterator<KopeteAccount> it( d->accounts ); it.current(); ++it )
-	{
-		if ( it.current()->autoLogin() )
-			it.current()->connect();
-	}
-}
-
-void KopeteAccountManager::notifyAccountReady( KopeteAccount *account )
-{
-	//kdDebug(14010) << k_funcinfo << account->accountId() << endl;
-	emit accountReady( account );
-	d->accounts.sort();
-
-	// Connect to the account's status changed signal
-	connect(account->myself(), SIGNAL(onlineStatusChanged(KopeteContact *,
-			const KopeteOnlineStatus &, const KopeteOnlineStatus &)),
-		this, SLOT(slotAccountOnlineStatusChanged(KopeteContact *,
-			const KopeteOnlineStatus &, const KopeteOnlineStatus &)));
-}
-
-void KopeteAccountManager::slotAccountOnlineStatusChanged(KopeteContact *c,
-	const KopeteOnlineStatus &oldStatus, const KopeteOnlineStatus &newStatus)
-{
-	KopeteAccount *account = c->account();
+	Account *account = c->account();
 	if (!account)
 		return;
 
@@ -352,5 +369,8 @@ void KopeteAccountManager::slotAccountOnlineStatusChanged(KopeteContact *c,
 	emit accountOnlineStatusChanged(account, oldStatus, newStatus);
 }
 
+} //END namespace Kopete
+
 #include "kopeteaccountmanager.moc"
 // vim: set noet ts=4 sts=4 sw=4:
+// kate: tab-width 4; indent-mode csands;
