@@ -4,6 +4,7 @@
     Kopete Latex Plugin
 
     Copyright (c) 2004 by Duncan Mac-Vicar Prett   <duncan@kde.org>
+    Copyright (c) 2004-2005 by Olivier Goffart  <ogoffart@kde. org>
 
     Kopete    (c) 2001-2004 by the Kopete developers  <kopete-devel@kde.org>
 
@@ -23,20 +24,22 @@
 #include <qobject.h>
 #include <qstring.h>
 
+#include <ktempfile.h>
+
 #include "kopetemessage.h"
 #include "kopeteplugin.h"
 
 class QStringList;
 class QString;
 
-class KopeteMessage;
-class LatexConfig;
+
+namespace Kopete { class Message; }
 
 /**
   * @author Duncan Mac-Vicar Prett
   */
 
-class LatexPlugin : public KopetePlugin
+class LatexPlugin : public Kopete::Plugin
 {
 	Q_OBJECT
 
@@ -47,14 +50,20 @@ public:
 	~LatexPlugin();
 
 public slots:
-	void slotHandleLatex( KopeteMessage& msg );
 	void slotSettingsChanged();
-
+	void slotMessageAboutToShow( Kopete::Message& msg );
+	void slotMessageAboutToSend( Kopete::Message& msg );
 
 private:
+	/**
+	 * gives a latex formula, and return the filename of the file where the latex is stored.
+     */
+	QString handleLatex(const QString &latex);
+	
 	static LatexPlugin* s_pluginStatic;
 	QString m_convScript;
-	LatexConfig *m_config;
+	bool mMagickNotFoundShown;
+	QPtrList<KTempFile> m_tempFiles;
 };
 
 #endif
