@@ -1,4 +1,4 @@
-/* $Id: common.c,v 1.1.4.1 2004/10/29 13:41:21 gj Exp $ */
+/* $Id: common.c,v 1.5 2004/12/07 12:09:31 gj Exp $ */
 
 /*
  *  (C) Copyright 2001-2002 Wojtek Kaniewski <wojtekka@irc.pl>
@@ -13,9 +13,10 @@
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *  GNU Lesser General Public License for more details.
  *
- *  You should have received a copy of the GNU Lesser General Public
- *  License along with this program; if not, write to the Free Software
- *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program; if not, write to the Free Software
+ *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
+ *  02111-1307, USA.
  */
 
 #include <sys/types.h>
@@ -38,7 +39,7 @@
 
 #include "libgadu.h"
 
-FILE *gg_debug_file = NULL;
+FILE *gg_debug_file;
 
 #ifndef GG_DEBUG_DISABLE
 
@@ -54,7 +55,7 @@ void gg_debug(int level, const char *format, ...)
 {
 	va_list ap;
 	int old_errno = errno;
-	
+
 	if (gg_debug_handler) {
 		va_start(ap, format);
 		(*gg_debug_handler)(level, format, ap);
@@ -62,7 +63,7 @@ void gg_debug(int level, const char *format, ...)
 
 		goto cleanup;
 	}
-	
+
 	if ((gg_debug_level & level)) {
 		va_start(ap, format);
 		vfprintf((gg_debug_file) ? gg_debug_file : stderr, format, ap);
@@ -93,7 +94,7 @@ char *gg_vsaprintf(const char *format, va_list ap)
         int size = 0;
 	const char *start;
 	char *buf = NULL;
-	
+
 #ifdef __GG_LIBGADU_HAVE_VA_COPY
 	va_list aq;
 
@@ -106,13 +107,13 @@ char *gg_vsaprintf(const char *format, va_list ap)
 #  endif
 #endif
 
-	start = format; 
+	start = format;
 
 #ifndef __GG_LIBGADU_HAVE_C99_VSNPRINTF
 	{
 		int res;
 		char *tmp;
-		
+
 		size = 128;
 		do {
 			size *= 2;
@@ -127,7 +128,7 @@ char *gg_vsaprintf(const char *format, va_list ap)
 #else
 	{
 		char tmp[2];
-		
+
 		/* libce Solarisa przy buforze NULL zawsze zwracaj -1, wic
 		 * musimy poda co istniejcego jako cel printf()owania. */
 		size = vsnprintf(tmp, sizeof(tmp), format, ap);
@@ -137,7 +138,7 @@ char *gg_vsaprintf(const char *format, va_list ap)
 #endif
 
 	format = start;
-	
+
 #ifdef __GG_LIBGADU_HAVE_VA_COPY
 	vsnprintf(buf, size + 1, format, aq);
 	va_end(aq);
@@ -149,7 +150,7 @@ char *gg_vsaprintf(const char *format, va_list ap)
 	vsnprintf(buf, size + 1, format, ap);
 #  endif
 #endif
-	
+
 	return buf;
 }
 
@@ -179,15 +180,15 @@ char *gg_saprintf(const char *format, ...)
 
 /*
  * gg_get_line() // funkcja pomocnicza
- * 
+ *
  * podaje kolejn lini z bufora tekstowego. niszczy go bezpowrotnie, dzielc
  * na kolejne stringi. zdarza si, nie ma potrzeby pisania funkcji dublujcej
  * bufor eby tylko mie nieruszone dane wejciowe, skoro i tak nie bd nam
  * poniej potrzebne. obcina `\r\n'.
- * 
+ *
  *  - ptr - wskanik do zmiennej, ktra przechowuje aktualn pozycj
  *    w przemiatanym buforze
- * 
+ *
  * wskanik do kolejnej linii tekstu lub NULL, jeli to ju koniec bufora.
  */
 char *gg_get_line(char **ptr)
@@ -232,7 +233,7 @@ int gg_connect(void *addr, int port, int async)
         struct sockaddr_in myaddr;
 
 	gg_debug(GG_DEBUG_FUNCTION, "** gg_connect(%s, %d, %d);\n", inet_ntoa(*a), port, async);
-	
+
 	if ((sock = socket(AF_INET, SOCK_STREAM, IPPROTO_TCP)) == -1) {
 		gg_debug(GG_DEBUG_MISC, "// gg_connect() socket() failed (errno=%d, %s)\n", errno, strerror(errno));
 		return -1;
@@ -242,7 +243,7 @@ int gg_connect(void *addr, int port, int async)
         myaddr.sin_family = AF_INET;
 
         myaddr.sin_addr.s_addr = gg_local_ip;
-        
+
         if(bind(sock, (struct sockaddr *) &myaddr, sizeof(myaddr)) == -1) {
                 gg_debug(GG_DEBUG_MISC, "// gg_connect() bind() failed (errno=%d, %s)\n", errno, strerror(errno));
                 return -1;
@@ -267,7 +268,7 @@ int gg_connect(void *addr, int port, int async)
 	sin.sin_port = htons(port);
 	sin.sin_family = AF_INET;
 	sin.sin_addr.s_addr = a->s_addr;
-	
+
 	if (connect(sock, (struct sockaddr*) &sin, sizeof(sin)) == -1) {
 		if (errno && (!async || errno != EINPROGRESS)) {
 			gg_debug(GG_DEBUG_MISC, "// gg_connect() connect() failed (errno=%d, %s)\n", errno, strerror(errno));
@@ -276,7 +277,7 @@ int gg_connect(void *addr, int port, int async)
 		}
 		gg_debug(GG_DEBUG_MISC, "// gg_connect() connect() in progress\n");
 	}
-	
+
 	return sock;
 }
 
@@ -411,7 +412,7 @@ int gg_http_hash(const char *format, ...)
 		} else {
 			if (!(arg = va_arg(ap, unsigned char*)))
 				arg = "";
-		}	
+		}
 
 		i = 0;
 		while ((c = (int) arg[i++]) != 0) {
@@ -445,12 +446,12 @@ struct in_addr *gg_gethostbyname(const char *hostname)
 	int h_errnop, ret;
 	size_t buflen = 1024;
 	int new_errno;
-	
+
 	new_errno = ENOMEM;
-	
+
 	if (!(addr = malloc(sizeof(struct in_addr))))
 		goto cleanup;
-	
+
 	if (!(hp = calloc(1, sizeof(*hp))))
 		goto cleanup;
 
@@ -458,39 +459,39 @@ struct in_addr *gg_gethostbyname(const char *hostname)
 		goto cleanup;
 
 	tmpbuf = buf;
-	
+
 	while ((ret = gethostbyname_r(hostname, hp, buf, buflen, &hp2, &h_errnop)) == ERANGE) {
 		buflen *= 2;
-		
+
 		if (!(tmpbuf = realloc(buf, buflen)))
 			break;
-		
+
 		buf = tmpbuf;
 	}
-	
+
 	if (ret)
 		new_errno = h_errnop;
 
 	if (ret || !hp2 || !tmpbuf)
 		goto cleanup;
-	
+
 	memcpy(addr, hp->h_addr, sizeof(struct in_addr));
-	
+
 	free(buf);
 	free(hp);
-	
+
 	return addr;
-	
+
 cleanup:
 	errno = new_errno;
-	
+
 	if (addr)
 		free(addr);
 	if (hp)
 		free(hp);
 	if (buf)
 		free(buf);
-	
+
 	return NULL;
 #else
 	struct hostent *hp;
@@ -506,7 +507,7 @@ cleanup:
 	memcpy(addr, hp->h_addr, sizeof(struct in_addr));
 
 	return addr;
-	
+
 cleanup:
 	if (addr)
 		free(addr);
@@ -534,7 +535,7 @@ struct gg_win32_thread *gg_win32_threads = 0;
  * jeli na win32 przy poczeniach synchronicznych zapamitamy w jakim
  * wtku uruchomilimy funkcj, ktra si z czymkolwiek czy, to z osobnego
  * wtku moemy anulowa poczenie poprzez gg_win32_thread_socket(watek, -1);
- * 
+ *
  * - thread_id - id wtku. jeli jest rwne 0, brany jest aktualny wtek,
  *               jeli rwne -1, usuwa wpis o podanym sockecie.
  * - socket - deskryptor gniazda. jeli rwne 0, zwraca deskryptor gniazda
@@ -551,7 +552,7 @@ int gg_win32_thread_socket(int thread_id, int socket)
 
 	if (!thread_id)
 		thread_id = GetCurrentThreadId();
-	
+
 	while (wsk) {
 		if ((thread_id == -1 && wsk->socket == socket) || wsk->id == thread_id) {
 			if (close) {
@@ -569,23 +570,23 @@ int gg_win32_thread_socket(int thread_id, int socket)
 				return socket;
 			}
                }
-		
+
                p_wsk = &(wsk->next);
                wsk = wsk->next;
         }
-	
+
         if (close && socket != -1)
 		closesocket(socket);
         if (close || !socket)
 		return 0;
-	
+
         /* Dodaje nowy element */
         wsk = malloc(sizeof(gg_win32_thread));
         wsk->id = thread_id;
         wsk->socket = socket;
         wsk->next = 0;
         *p_wsk = wsk;
-	
+
         return socket;
 }
 
@@ -607,13 +608,13 @@ char *gg_base64_encode(const char *buf)
 {
 	char *out, *res;
 	int i = 0, j = 0, k = 0, len = strlen(buf);
-	
+
 	res = out = malloc((len / 3 + 1) * 4 + 2);
 
 	if (!res)
 		return NULL;
-	
-	while (j <= len) {
+
+	while (j < len) {
 		switch (i % 4) {
 			case 0:
 				k = (buf[j] & 252) >> 2;
@@ -645,9 +646,9 @@ char *gg_base64_encode(const char *buf)
 	if (i % 4)
 		for (j = 0; j < 4 - (i % 4); j++, out++)
 			*out = '=';
-	
+
 	*out = 0;
-	
+
 	return res;
 }
 
@@ -668,7 +669,7 @@ char *gg_base64_decode(const char *buf)
 
 	if (!buf)
 		return NULL;
-	
+
 	save = res = calloc(1, (strlen(buf) / 4 + 1) * 3 + 2);
 
 	if (!save)
@@ -705,7 +706,7 @@ char *gg_base64_decode(const char *buf)
 		index %= 4;
 	}
 	*res = 0;
-	
+
 	return save;
 }
 
@@ -713,7 +714,7 @@ char *gg_base64_decode(const char *buf)
  * gg_proxy_auth() // funkcja wewntrzna
  *
  * tworzy nagwek autoryzacji dla proxy.
- * 
+ *
  * zaalokowany tekst lub NULL, jeli proxy nie jest wczone lub nie wymaga
  * autoryzacji.
  */
@@ -721,7 +722,7 @@ char *gg_proxy_auth()
 {
 	char *tmp, *enc, *out;
 	unsigned int tmp_size;
-	
+
 	if (!gg_proxy_enabled || !gg_proxy_username || !gg_proxy_password)
 		return NULL;
 
@@ -734,14 +735,14 @@ char *gg_proxy_auth()
 		free(tmp);
 		return NULL;
 	}
-	
+
 	free(tmp);
 
 	if (!(out = malloc(strlen(enc) + 40))) {
 		free(enc);
 		return NULL;
 	}
-	
+
 	snprintf(out, strlen(enc) + 40,  "Proxy-Authorization: Basic %s\r\n", enc);
 
 	free(enc);
