@@ -1,9 +1,9 @@
 /*
     msnmessagemanager.cpp - MSN Message Manager
 
-    Copyright (c) 2002-2004 by Olivier Goffart        <ogoffart@tiscalinet.be>
+    Copyright (c) 2002-2005 by Olivier Goffart        <ogoffart at kde.org>
 
-    Kopete    (c) 2002-2004 by the Kopete developers  <kopete-devel@kde.org>
+    Kopete    (c) 2002-2005 by the Kopete developers  <kopete-devel@kde.org>
 
     *************************************************************************
     *                                                                       *
@@ -34,6 +34,7 @@
 #include <krun.h>
 
 #include "kopetecontactaction.h"
+#include "kopetemetacontact.h"
 #include "kopetecontactlist.h"
 #include "kopetemessagemanagerfactory.h"
 #include "kopeteuiglobal.h"
@@ -49,23 +50,24 @@
 #include "msndebugrawcmddlg.h"
 #endif
 
-MSNMessageManager::MSNMessageManager( KopeteProtocol *protocol, const KopeteContact *user,
-	KopeteContactPtrList others, const char *name )
-: KopeteMessageManager( user, others, protocol, 0, name )
+MSNChatSession::MSNChatSession( Kopete::Protocol *protocol, const Kopete::Contact *user,
+	Kopete::ContactPtrList others, const char *name )
+: Kopete::ChatSession( user, others, protocol,  name )
 {
-	KopeteMessageManagerFactory::factory()->addKopeteMessageManager( this );
+	Kopete::ChatSessionManager::self()->registerChatSession( this );
 	m_chatService = 0l;
 //	m_msgQueued = 0L;
+	m_newSession = true;
 
 	setInstance(protocol->instance());
 
-	connect( this, SIGNAL( messageSent( KopeteMessage&,
-		KopeteMessageManager* ) ),
-		this, SLOT( slotMessageSent( KopeteMessage&,
-		KopeteMessageManager* ) ) );
+	connect( this, SIGNAL( messageSent( Kopete::Message&,
+		Kopete::ChatSession* ) ),
+		this, SLOT( slotMessageSent( Kopete::Message&,
+		Kopete::ChatSession* ) ) );
 
-	connect( this, SIGNAL( invitation(MSNInvitation*& ,  const QString & , long unsigned int , MSNMessageManager*  , MSNContact*  ) ) ,
-		protocol,  SIGNAL( invitation(MSNInvitation*& ,  const QString & , long unsigned int , MSNMessageManager*  , MSNContact*  ) ) );
+	connect( this, SIGNAL( invitation(MSNInvitation*& ,  const QString & , long unsigned int , MSNChatSession*  , MSNContact*  ) ) ,
+		protocol,  SIGNAL( invitation(MSNInvitation*& ,  const QString & , long unsigned int , MSNChatSession*  , MSNContact*  ) ) );
 
 
 	m_actionInvite = new KActionMenu( i18n( "&Invite" ), actionCollection() , "msnInvite" );
@@ -80,17 +82,18 @@ MSNMessageManager::MSNMessageManager( KopeteProtocol *protocol, const KopeteCont
 
 	if ( !c->object().isEmpty() )
 	{
+		
 		connect( c, SIGNAL( displayPictureChanged() ), this, SLOT( slotDisplayPictureChanged() ) );
 		m_image = new QLabel( 0L, "kde toolbar widget" );
 		new KWidgetAction( m_image, i18n( "MSN Display Picture" ), 0, this, SLOT( slotRequestPicture() ), actionCollection(), "msnDisplayPicture" );
-		if(c->displayPicture())
+		if(c->hasProperty(Kopete::Global::Properties::self()->photo().key())  )
 		{
 			//if the view doesn't exist yet, we will be unable to get the size of the toolbar
 			// so when the view will exist, we will show the displaypicture.
 			//How to know when a our view is created?  We can't.
 			// but chances are the next created view will be for this KMM
 			// And if it is not?  never mind. the icon will just be sized 22x22
-			connect( KopeteMessageManagerFactory::factory() , SIGNAL(viewActivated(KopeteView* )) , this, SLOT(slotDisplayPictureChanged()) );
+			connect( Kopete::ChatSessionManager::self() , SIGNAL(viewActivated(KopeteView* )) , this, SLOT(slotDisplayPictureChanged()) );
 			//it's viewActivated and not viewCreated because the view get his mainwindow only when it is shown.
 		}
 	}
@@ -100,9 +103,11 @@ MSNMessageManager::MSNMessageManager( KopeteProtocol *protocol, const KopeteCont
 	}
 
 	setXMLFile("msnchatui.rc");
+	
+	setMayInvite( true );
 }
 
-MSNMessageManager::~MSNMessageManager()
+MSNChatSession::~MSNChatSession()
 {
 	delete m_image;
 	//force to disconnect the switchboard
@@ -118,9 +123,14 @@ MSNMessageManager::~MSNMessageManager()
 	}
 }
 
-void MSNMessageManager::createChat( const QString &handle,
+void MSNChatSession::createChat( const QString &handle,
 	const QString &address, const QString &auth, const QString &ID )
 {
+	/** disabled because i don't want to reopen a chatwindow if we just closed it
+	 * and the contact take much time to type his message
+	 m_newSession= !(ID.isEmpty());
+	*/
+	
 	if( m_chatService )
 	{
 		kdDebug(14140) << k_funcinfo << "Service already exists, disconnect them." << endl;
@@ -130,8 +140,8 @@ void MSNMessageManager::createChat( const QString &handle,
 //	uncomment this line if you don't want to the peer know when you close the window
 //	setCanBeDeleted( false );
 
-	m_chatService = new MSNSwitchBoardSocket( static_cast<MSNAccount*>( user()->account() ) , this);
-	m_chatService->setHandle( user()->account()->accountId() );
+	m_chatService = new MSNSwitchBoardSocket( static_cast<MSNAccount*>( myself()->account() ) , this);
+	m_chatService->setHandle( myself()->account()->accountId() );
 	m_chatService->setMsgHandle( handle );
 	m_chatService->connectToSwitchBoard( ID, address, auth );
 
@@ -139,13 +149,13 @@ void MSNMessageManager::createChat( const QString &handle,
 		this, SLOT( slotUserJoined(const QString&,const QString&,bool) ) );
 	connect( m_chatService, SIGNAL( userLeft(const QString&,const QString&)),
 		this, SLOT( slotUserLeft(const QString&,const QString&) ) );
-	connect( m_chatService, SIGNAL( msgReceived( KopeteMessage & ) ),
-		this, SLOT( slotMessageReceived( KopeteMessage & ) ) );
+	connect( m_chatService, SIGNAL( msgReceived( Kopete::Message & ) ),
+		this, SLOT( slotMessageReceived( Kopete::Message & ) ) );
 	connect( m_chatService, SIGNAL( switchBoardClosed() ),
 		this, SLOT( slotSwitchBoardClosed() ) );
 	connect( m_chatService, SIGNAL( receivedTypingMsg( const QString &, bool ) ),
 		this, SLOT( receivedTypingMsg( const QString &, bool ) ) );
-	connect( this, SIGNAL( typingMsg( bool ) ),
+	connect( this, SIGNAL( myselfTyping( bool ) ),
 		m_chatService, SLOT( sendTypingMsg( bool ) ) );
 	connect( m_chatService, SIGNAL( msgAcknowledgement(unsigned int, bool) ),
 		this, SLOT( slotAcknowledgement(unsigned int, bool) ) );
@@ -153,15 +163,14 @@ void MSNMessageManager::createChat( const QString &handle,
 		this, SLOT( slotInvitation( const QString&, const QString& ) ) );
 }
 
-void MSNMessageManager::slotUserJoined( const QString &handle, const QString &publicName, bool IRO )
+void MSNChatSession::slotUserJoined( const QString &handle, const QString &publicName, bool IRO )
 {
 	if( !account()->contacts()[ handle ] )
-		account()->addContact( handle, publicName, 0L, KopeteAccount::DontChangeKABC, QString::null, true);
+		account()->addContact( handle, QString::null, 0L, Kopete::Account::Temporary);
 
 	MSNContact *c = static_cast<MSNContact*>( account()->contacts()[ handle ] );
 
-	if( c->property( Kopete::Global::Properties::self()->nickName()).value().toString() != publicName)
-		c->rename(publicName);
+	c->setProperty( Kopete::Global::Properties::self()->nickName() , publicName);
 
 	addContact(c , IRO); // don't show notificaions when we join wesalef
 	if(!m_messagesQueue.empty() || !m_invitations.isEmpty())
@@ -169,30 +178,30 @@ void MSNMessageManager::slotUserJoined( const QString &handle, const QString &pu
 
 	KConfig *config = KGlobal::config();
 	config->setGroup( "MSN" );
-	if ( members().count()==1 && config->readBoolEntry( "AutoDownloadPicture", true ) && !c->object().isEmpty() && !c->displayPicture())
+	if ( members().count()==1 && config->readBoolEntry( "AutoDownloadPicture", true ) && !c->object().isEmpty() && !c->hasProperty(Kopete::Global::Properties::self()->photo().key()))
 		slotRequestPicture();
 }
 
-void MSNMessageManager::slotUserLeft( const QString &handle, const QString& reason )
+void MSNChatSession::slotUserLeft( const QString &handle, const QString& reason )
 {
-	MSNContact *c = static_cast<MSNContact*>( user()->account()->contacts()[ handle ] );
+	MSNContact *c = static_cast<MSNContact*>( myself()->account()->contacts()[ handle ] );
 	if(c)
 		removeContact(c, reason );
 }
 
 
 
-void MSNMessageManager::slotSwitchBoardClosed()
+void MSNChatSession::slotSwitchBoardClosed()
 {
-	//kdDebug(14140) << "MSNMessageManager::slotSwitchBoardClosed"  << endl;
+	//kdDebug(14140) << "MSNChatSession::slotSwitchBoardClosed"  << endl;
 	m_chatService->deleteLater();
 	m_chatService=0l;
 
-	for ( QMap<unsigned int , KopeteMessage>::iterator it = m_messagesSent.begin(); it!=m_messagesSent.end(); it = m_messagesSent.begin() )
+	for ( QMap<unsigned int , Kopete::Message>::iterator it = m_messagesSent.begin(); it!=m_messagesSent.end(); it = m_messagesSent.begin() )
 	{
-		KopeteMessage m=it.data();
+		Kopete::Message m=it.data();
 		QString body=i18n("The following message has not been sent correctly: \n%1").arg(m.plainBody());
-		KopeteMessage msg = KopeteMessage(m.to().first() , members() , body , KopeteMessage::Internal, KopeteMessage::PlainText);
+		Kopete::Message msg = Kopete::Message(m.to().first() , members() , body , Kopete::Message::Internal, Kopete::Message::PlainText);
 		appendMessage(msg);
 
 		m_messagesSent.remove(it);
@@ -202,8 +211,9 @@ void MSNMessageManager::slotSwitchBoardClosed()
 		setCanBeDeleted( true );
 }
 
-void MSNMessageManager::slotMessageSent(KopeteMessage &message,KopeteMessageManager *)
+void MSNChatSession::slotMessageSent(Kopete::Message &message,Kopete::ChatSession *)
 {
+	m_newSession=false;
  	if(m_chatService)
 	{
 		int id = m_chatService->sendMsg(message);
@@ -217,25 +227,31 @@ void MSNMessageManager::slotMessageSent(KopeteMessage &message,KopeteMessageMana
 			//FIXME:  tell the what window the message has been processed. but we havent't sent it 
 			messageSucceeded();  //that should stop the blonking icon.
 		}
+		else if( id == -3) //the message has been sent as an immge
+		{
+			appendMessage(message); 
+			messageSucceeded();
+		}
 		else
 		{
 			m_messagesSent.insert( id, message );
 			message.setBg(QColor()); // clear the bgColor
-			message.setBody(message.plainBody() , KopeteMessage::PlainText ); //clear every custom tag which are not sent
+			message.setBody(message.plainBody() , Kopete::Message::PlainText ); //clear every custom tag which are not sent
 			appendMessage(message); // send the own msg to chat window
 		}
 	}
 	else // There's no switchboard available, so we must create a new one!
 	{
-		static_cast<MSNAccount*>( user()->account() )->slotStartChatSession( message.to().first()->contactId() );
+		static_cast<MSNAccount*>( myself()->account() )->slotStartChatSession( message.to().first()->contactId() );
 		m_messagesQueue.append(message);
 //		sendMessageQueue();
-		//m_msgQueued=new KopeteMessage(message);
+		//m_msgQueued=new Kopete::Message(message);
 	}
 }
 
-void MSNMessageManager::slotMessageReceived( KopeteMessage &msg )
+void MSNChatSession::slotMessageReceived( Kopete::Message &msg )
 {
+	m_newSession=false;
 	if( msg.plainBody().startsWith( "AutoMessage: " ) )
 	{
 		//FIXME: HardCodded color are not so good
@@ -254,8 +270,8 @@ void MSNMessageManager::slotMessageReceived( KopeteMessage &msg )
 			m_awayMessageTime.elapsed() > 1000 * config->readNumEntry( "AwayMessagesSeconds", 90 ) )  )
 		{
 			// Don't translate "Auto-Message:" This string is caught by MSN Plus! (and also by kopete now)
-			KopeteMessage msg2( user(), members(),
-				"AutoMessage: " + static_cast<MSNAccount *>( account() )->awayReason(), KopeteMessage::Outbound );
+			Kopete::Message msg2( myself(), members(),
+				"AutoMessage: " + static_cast<MSNAccount *>( account() )->awayReason(), Kopete::Message::Outbound );
 			msg2.setFg( QColor( "SlateGray3" ) );
 			QFont f;
 			f.setItalic( true );
@@ -266,7 +282,7 @@ void MSNMessageManager::slotMessageReceived( KopeteMessage &msg )
 	}
 }
 
-void MSNMessageManager::slotActionInviteAboutToShow()
+void MSNChatSession::slotActionInviteAboutToShow()
 {
 	// We can't simply insert  KAction in this menu bebause we don't know when to delete them.
 	//  items inserted with insert items are automatically deleted when we call clear
@@ -277,13 +293,13 @@ void MSNMessageManager::slotActionInviteAboutToShow()
 	m_actionInvite->popupMenu()->clear();
 
 	
-	QDictIterator<KopeteContact> it( account()->contacts() );
+	QDictIterator<Kopete::Contact> it( account()->contacts() );
 	for( ; it.current(); ++it )
 	{
-		if( !members().contains( it.current() ) && it.current()->isOnline() && it.current() != user() )
+		if( !members().contains( it.current() ) && it.current()->isOnline() && it.current() != myself() )
 		{
 			KAction *a=new KopeteContactAction( it.current(), this,
-				SLOT( slotInviteContact( KopeteContact * ) ), m_actionInvite );
+				SLOT( slotInviteContact( Kopete::Contact * ) ), m_actionInvite );
 			m_actionInvite->insert( a );
 			m_inviteactions.append( a ) ;
 		}
@@ -293,23 +309,28 @@ void MSNMessageManager::slotActionInviteAboutToShow()
 	m_inviteactions.append( b ) ;
 }
 
-void MSNMessageManager::slotCloseSession()
+void MSNChatSession::slotCloseSession()
 {
 	kdDebug(14140) << k_funcinfo  << m_chatService <<endl;
 	if(m_chatService)
 		m_chatService->slotCloseSession();
 }
 
-void MSNMessageManager::slotInviteContact( KopeteContact *contact )
+void MSNChatSession::slotInviteContact( Kopete::Contact *contact )
 {
-	if( m_chatService )
-		m_chatService->slotInviteContact( contact->contactId() );
-	else
-		static_cast<MSNAccount*>( user()->account() )->slotStartChatSession( contact->contactId() );
+	if(contact)
+		inviteContact( contact->contactId() );
 }
 
+void MSNChatSession::inviteContact(const QString &contactId)
+{
+	if( m_chatService )
+		m_chatService->slotInviteContact( contactId );
+	else
+		static_cast<MSNAccount*>( myself()->account() )->slotStartChatSession( contactId );
+}
 
-void MSNMessageManager::slotInviteOtherContact()
+void MSNChatSession::slotInviteOtherContact()
 {
 	bool ok;
 	QString handle = KInputDialog::getText(i18n( "MSN Plugin" ),
@@ -325,22 +346,19 @@ void MSNMessageManager::slotInviteOtherContact()
 			return;
 	}
 
-	if( m_chatService )
-		m_chatService->slotInviteContact( handle );
-	else
-		static_cast<MSNAccount*>( user()->account() )->slotStartChatSession( handle );
+	inviteContact(handle);
 }
 
 
-void MSNMessageManager::sendMessageQueue()
+void MSNChatSession::sendMessageQueue()
 {
 	if(!m_chatService)
 	{
 		kdDebug(14140) <<k_funcinfo << "Service doesn't exist" <<endl;
 		return;
 	}
-//	kdDebug(14140) << "MSNMessageManager::sendMessageQueue: " << m_messagesQueue.count() <<endl;
-	for ( QValueList<KopeteMessage>::iterator it = m_messagesQueue.begin(); it!=m_messagesQueue.end(); it = m_messagesQueue.begin() )
+//	kdDebug(14140) << "MSNChatSession::sendMessageQueue: " << m_messagesQueue.count() <<endl;
+	for ( QValueList<Kopete::Message>::iterator it = m_messagesQueue.begin(); it!=m_messagesQueue.end(); it = m_messagesQueue.begin() )
 	{
 		//m_chatService->sendMsg( *it)  ;
 		slotMessageSent(*it , this);
@@ -359,7 +377,7 @@ void MSNMessageManager::sendMessageQueue()
 	}
 }
 
-void MSNMessageManager::slotAcknowledgement(unsigned int id, bool ack)
+void MSNChatSession::slotAcknowledgement(unsigned int id, bool ack)
 {
 	if ( !m_messagesSent.contains( id ) )
 	{
@@ -369,9 +387,9 @@ void MSNMessageManager::slotAcknowledgement(unsigned int id, bool ack)
 
 	if ( !ack )
 	{
-		KopeteMessage m = m_messagesSent[ id ];
+		Kopete::Message m = m_messagesSent[ id ];
 		QString body = i18n( "The following message has not been sent correctly:\n%1" ).arg( m.plainBody() );
-		KopeteMessage msg = KopeteMessage( m.to().first(), members(), body, KopeteMessage::Internal, KopeteMessage::PlainText );
+		Kopete::Message msg = Kopete::Message( m.to().first(), members(), body, Kopete::Message::Internal, Kopete::Message::PlainText );
 		appendMessage( msg );
 		//stop the stupid animation
 		messageSucceeded();  
@@ -384,10 +402,10 @@ void MSNMessageManager::slotAcknowledgement(unsigned int id, bool ack)
 	m_messagesSent.remove( id );
 }
 
-void MSNMessageManager::slotInvitation(const QString &handle, const QString &msg)
+void MSNChatSession::slotInvitation(const QString &handle, const QString &msg)
 {
 	//FIXME! a contact from another account can send a file
-	MSNContact *c = static_cast<MSNContact*>( user()->account()->contacts()[ handle ] );
+	MSNContact *c = static_cast<MSNContact*>( myself()->account()->contacts()[ handle ] );
 	if(!c)
 		return;
 
@@ -404,7 +422,7 @@ void MSNMessageManager::slotInvitation(const QString &handle, const QString &msg
 	{
 		if( msg.contains(MSNFileTransferSocket::applicationID()) )
 		{
-			MSNFileTransferSocket *MFTS=new MSNFileTransferSocket(user()->account()->accountId(),c,true,this);
+			MSNFileTransferSocket *MFTS=new MSNFileTransferSocket(myself()->account()->accountId(),c,true,this);
 			connect(MFTS, SIGNAL( done(MSNInvitation*) ) , this , SLOT( invitationDone(MSNInvitation*) ));
 			m_invitations.insert( cookie  , MFTS);
 			MFTS->parseInvitation(msg);
@@ -430,7 +448,7 @@ void MSNMessageManager::slotInvitation(const QString &handle, const QString &msg
 					"%1 has sent an unimplemented invitation, the invitation was rejected.\n"
 					"The invitation was: %2" )
 						.arg( c->property( Kopete::Global::Properties::self()->nickName()).value().toString(), inviteName );
-				KopeteMessage tmpMsg = KopeteMessage( c , members() , body , KopeteMessage::Internal, KopeteMessage::PlainText);
+				Kopete::Message tmpMsg = Kopete::Message( c , members() , body , Kopete::Message::Internal, Kopete::Message::PlainText);
 				appendMessage(tmpMsg);
 
 				m_chatService->sendCommand( "MSG" , "N", true, MSNInvitation::unimplemented(cookie) );
@@ -439,7 +457,7 @@ void MSNMessageManager::slotInvitation(const QString &handle, const QString &msg
 	}
 }
 
-void MSNMessageManager::invitationDone(MSNInvitation* MFTS)
+void MSNChatSession::invitationDone(MSNInvitation* MFTS)
 {
 	kdDebug(14140) << k_funcinfo <<endl;
 	m_invitations.remove(MFTS->cookie());
@@ -449,7 +467,7 @@ void MSNMessageManager::invitationDone(MSNInvitation* MFTS)
 		setCanBeDeleted(true);
 }
 
-void MSNMessageManager::sendFile(const QString &fileLocation, const QString &/*fileName*/,
+void MSNChatSession::sendFile(const QString &fileLocation, const QString &/*fileName*/,
 	long unsigned int fileSize)
 {
 //	if(m_chatService)
@@ -463,8 +481,8 @@ void MSNMessageManager::sendFile(const QString &fileLocation, const QString &/*f
 			theFileName = fileName;
 		}*/
 
-		QPtrList<KopeteContact>contacts=members();
-		MSNFileTransferSocket *MFTS=new MSNFileTransferSocket(user()->account()->accountId(),contacts.first(), false,this);
+		QPtrList<Kopete::Contact>contacts=members();
+		MSNFileTransferSocket *MFTS=new MSNFileTransferSocket(myself()->account()->accountId(),contacts.first(), false,this);
 
 		//Call the setFile command to let the MFTS know what file we are sending
 		MFTS->setFile(fileLocation, fileSize);
@@ -473,7 +491,7 @@ void MSNMessageManager::sendFile(const QString &fileLocation, const QString &/*f
 //	}
 }
 
-void MSNMessageManager::initInvitation(MSNInvitation* invitation)
+void MSNChatSession::initInvitation(MSNInvitation* invitation)
 {
 	connect(invitation->object(), SIGNAL( done(MSNInvitation*) ) , this , SLOT( invitationDone(MSNInvitation*) ));
 	m_invitations.insert( invitation->cookie() , invitation);
@@ -485,19 +503,19 @@ void MSNMessageManager::initInvitation(MSNInvitation* invitation)
 	}
 	else
 	{
-		QPtrList<KopeteContact> mb=members();
+		QPtrList<Kopete::Contact> mb=members();
 		static_cast<MSNAccount*>( account() )->slotStartChatSession( mb.first()->contactId() );
 	}
 }
 
-void MSNMessageManager::slotRequestPicture()
+void MSNChatSession::slotRequestPicture()
 {
-	QPtrList<KopeteContact> mb=members();
+	QPtrList<Kopete::Contact> mb=members();
 	MSNContact *c = static_cast<MSNContact*>( mb.first() );
 	if(!c)
 	 return;
 	
-	if( !c->displayPicture())
+	if( !c->hasProperty(Kopete::Global::Properties::self()->photo().key()))
 	{
 		if(m_chatService)
 		{
@@ -509,17 +527,18 @@ void MSNMessageManager::slotRequestPicture()
 	}
 	else
 	{ //we already have the picture, just show it.
-		KRun::runURL( KURL::fromPathOrURL( c->displayPicture()->name() ), "image/png" );
+		KRun::runURL( KURL::fromPathOrURL( c->property(Kopete::Global::Properties::self()->photo()).value().toString() ), "image/png" );
 	}
 
 }
 
-void MSNMessageManager::slotDisplayPictureChanged()
+void MSNChatSession::slotDisplayPictureChanged()
 {
-	const MSNContact *c = static_cast<const MSNContact *>( members().getFirst() );
+	QPtrList<Kopete::Contact> mb=members();
+	MSNContact *c = static_cast<MSNContact *>( mb.first() );
 	if ( c && m_image )
 	{
-		if(c->displayPicture())
+		if(c->hasProperty(Kopete::Global::Properties::self()->photo().key()))
 		{
 			int sz=22;
 			// get the size of the toolbar were the aciton is plugged.
@@ -528,7 +547,7 @@ void MSNMessageManager::slotDisplayPictureChanged()
 			if(w)
 			{
 				//We connected that in the constructor.  we don't need to keep this slot active.
-				disconnect( KopeteMessageManagerFactory::factory() , SIGNAL(viewActivated(KopeteView* )) , this, SLOT(slotDisplayPictureChanged()) );
+				disconnect( Kopete::ChatSessionManager::self() , SIGNAL(viewActivated(KopeteView* )) , this, SLOT(slotDisplayPictureChanged()) );
 			
 				QPtrListIterator<KToolBar>  it=w->toolBarIterator() ;
 				KAction *imgAction=actionCollection()->action("msnDisplayPicture");
@@ -546,10 +565,17 @@ void MSNMessageManager::slotDisplayPictureChanged()
 					++it;
 				}
 			}
+			QString imgURL=c->property(Kopete::Global::Properties::self()->photo()).value().toString();
+			QImage scaledImg = QPixmap( imgURL ).convertToImage().smoothScale( sz, sz );
+			if(!scaledImg.isNull())
+				m_image->setPixmap( scaledImg );
+			else
+			{ //the image has maybe not been transfered correctly..  force to download again
+				c->removeProperty(Kopete::Global::Properties::self()->photo());
+				//slotDisplayPictureChanged(); //don't do that or we might end in a infinite loop
+			}
+			QToolTip::add( m_image, "<qt><img src=\"" + imgURL + "\"></qt>" );
 			
-			QImage scaledImg = QPixmap( c->displayPicture()->name() ).convertToImage().smoothScale( sz, sz );
-			m_image->setPixmap( scaledImg );
-			QToolTip::add( m_image, "<qt><img src=\"" + c->displayPicture()->name() + "\"></qt>" );
 		}
 		else 
 		{
@@ -561,7 +587,7 @@ void MSNMessageManager::slotDisplayPictureChanged()
 	}
 }
 
-void MSNMessageManager::slotDebugRawCommand()
+void MSNChatSession::slotDebugRawCommand()
 {
 #if !defined NDEBUG
 	if ( !m_chatService )
@@ -579,6 +605,27 @@ void MSNMessageManager::slotDebugRawCommand()
 }
 
 
+void MSNChatSession::receivedTypingMsg( const QString &contactId, bool b )
+{
+	MSNContact *c = dynamic_cast<MSNContact *>( account()->contacts()[ contactId ] );
+	if(c && m_newSession &&  !view(false))
+	{
+		//this was originaly in MSNAccount::slotCreateChat
+		KGlobal::config()->setGroup( "MSN" );
+		bool notifyNewChat = KGlobal::config()->readBoolEntry( "NotifyNewChat", false );
+		if (  notifyNewChat  )
+		{
+			// this internal message should open the window if they not exist
+			QString body = i18n( "%1 has started a chat with you" ).arg( c->metaContact()->displayName() );
+			Kopete::Message tmpMsg = Kopete::Message( c, members(), body, Kopete::Message::Internal, Kopete::Message::PlainText );
+			appendMessage( tmpMsg );
+		}
+	}
+	m_newSession=false;
+	if(c)
+		Kopete::ChatSession::receivedTypingMsg(c,b);
+}
+
 #include "msnmessagemanager.moc"
 
 // vim: set noet ts=4 sts=4 sw=4:
