@@ -127,7 +127,7 @@ ConnectWidget::ConnectWidget(QWidget *parent, const char *name, PPPStats *st)
   debug->setToggleButton(true);
   connect(debug, SIGNAL(clicked()), SIGNAL(toggleDebugWindow()));
 
-  cancel = new QPushButton(i18n("&Cancel"), this);
+  cancel = new KPushButton(KStdGuiItem::cancel(), this);
   cancel->setFocus();
   connect(cancel, SIGNAL(clicked()), SLOT(cancelbutton()));
 
@@ -168,6 +168,18 @@ ConnectWidget::~ConnectWidget() {
 }
 
 
+void ConnectWidget::disableButtons()
+{
+  debug->setEnabled(false);
+  cancel->setEnabled(false);
+}
+
+void ConnectWidget::enableButtons()
+{
+  debug->setEnabled(true);
+  cancel->setEnabled(true);
+}
+
 void ConnectWidget::preinit() {
   // this is all just to keep the GUI nice and snappy ....
   // you have to see to believe ...
@@ -175,7 +187,6 @@ void ConnectWidget::preinit() {
   inittimer->start(100);
 }
 
-
 void ConnectWidget::init() {
   gpppdata.setpppdError(0);
   inittimer->stop();
@@ -204,6 +215,9 @@ void ConnectWidget::init() {
 
   kapp->processEvents();
 
+  // signal other applications that we are about to get connected
+  kapp->dcopClient()->emitDCOPSignal("KpppIface", "aboutToConnect()", QByteArray());
+
   // run the "before-connect" command
   if (!gpppdata.command_before_connect().isEmpty()) {
     messg->setText(i18n("Running pre-startup command..."));
@@ -353,7 +367,7 @@ void ConnectWidget::timerEvent(QTimerEvent *) {
 
   if(vmain == 4) {
     if(!expecting) {
-      if(!gpppdata.waitForDialTone()) {
+      if(!gpppdata.waitForDialTone() || gpppdata.waitCallback()) {
 	QString msg = i18n("Turning off dial tone waiting...");
 	messg->setText(msg);
 	emit debugMessage(msg);
@@ -372,6 +386,15 @@ void ConnectWidget::timerEvent(QTimerEvent *) {
       timeout_timer->stop();
       timeout_timer->start(gpppdata.modemTimeout()*1000);
 
+      if(gpppdata.waitCallback()) {
+        QString msg = i18n("Waiting for callback...");
+        messg->setText(msg);
+        emit debugMessage(msg);
+        setExpect(gpppdata.modemRingResp());
+        vmain = 102;
+        return;
+      }
+
       QStringList &plist = gpppdata.phonenumbers();
       QString bmarg= gpppdata.dialPrefix();
       bmarg += *plist.at(dialnumber);
@@ -424,6 +447,7 @@ void ConnectWidget::timerEvent(QTimerEvent *) {
       Modem::modem->setDataMode(false);
       vmain = 0;
       substate = -1;
+      gpppdata.setWaitCallback(false);
       return;
     }
 
@@ -433,15 +457,38 @@ void ConnectWidget::timerEvent(QTimerEvent *) {
       messg->setText(i18n("No Dial Tone"));
       vmain = 20;
       Modem::modem->unlockdevice();
+      gpppdata.setWaitCallback(false);
       return;
     }
 
     if(readbuffer.contains(gpppdata.modemNoCarrierResp())) {
-      timeout_timer->stop();
+      if (gpppdata.get_redial_on_nocarrier()) {
+        timeout_timer->stop();
+        timeout_timer->start(gpppdata.modemTimeout()*1000);
 
-      messg->setText(i18n("No Carrier"));
-      vmain = 20;
-      Modem::modem->unlockdevice();
+        if(gpppdata.busyWait() > 0) {
+	  QString bm = i18n("No carrier. Waiting: %1 seconds").arg(gpppdata.busyWait());
+	  messg->setText(bm);
+	  emit debugMessage(bm);
+
+	  pausing = true;
+
+	  pausetimer->start(gpppdata.busyWait()*1000, true);
+	  timeout_timer->stop();
+        }
+
+        Modem::modem->setDataMode(false);
+        vmain = 0;
+        substate = -1;
+        return;
+      } else {
+        timeout_timer->stop();
+
+        messg->setText(i18n("No Carrier"));
+        vmain = 20;
+        Modem::modem->unlockdevice();
+        gpppdata.setWaitCallback(false);
+      }
       return;
     }
 
@@ -458,6 +505,7 @@ void ConnectWidget::timerEvent(QTimerEvent *) {
                               "Do NOT connect this modem to a digital phone "
 			      "line or the modem could get permanently "
 			      "damaged"));
+      gpppdata.setWaitCallback(false);
       return;
     }
 
@@ -478,6 +526,16 @@ void ConnectWidget::timerEvent(QTimerEvent *) {
     }
   }
 
+  // send answer on callback phase
+  if(vmain == 102) {
+    if(!expecting) {
+      writeline(gpppdata.modemAnswerStr());
+      setExpect(gpppdata.modemConnectResp());
+      vmain = 100;
+      return;
+    }
+  }
+
   // execute the script
   if(vmain == 2) {
     if(!expecting && !pausing && !scanning) {
@@ -760,7 +818,7 @@ void ConnectWidget::timerEvent(QTimerEvent *) {
 		bm += i18n("ERROR: Nested too deep, ignored.");
 		vmain=20;
 		cancelbutton();
-	        KMessageBox::error(0, i18n("Loops nested too deeply!"));
+	        KMessageBox::error(0, i18n("Loops nested too deeply."));
 	} else {
         	setExpect(scriptArgument);
 		loopstartindex[loopnest] = scriptindex;
@@ -776,7 +834,7 @@ void ConnectWidget::timerEvent(QTimerEvent *) {
       if (scriptCommand == "LoopEnd") {
         QString bm = i18n("Loop End %1").arg(scriptArgument);
 	if ( loopnest <= 0 ) {
-		bm = i18n("LoopEnd without matching Start! Line: %1").arg(bm);
+		bm = i18n("LoopEnd without matching Start. Line: %1").arg(bm);
 		vmain=20;
 		cancelbutton();
 	        KMessageBox::error(0, bm);
@@ -834,7 +892,7 @@ void ConnectWidget::timerEvent(QTimerEvent *) {
 	if (termwindow) {
 	  delete termwindow;
 	  termwindow = 0L;
-	  this->show();
+	  show();
 	} else {
 	  termwindow = new LoginTerm(0L, 0L);
 	  hide();
@@ -877,7 +935,8 @@ void ConnectWidget::timerEvent(QTimerEvent *) {
 	// starting pppd wasn't successful. Error messages were
 	// handled by execppp();
 	if_timeout_timer->stop();
-	this->hide();
+
+	hide();
 	messg->setText("");
 	p_kppp->quit_b->setFocus();
 	p_kppp->show();
@@ -897,6 +956,7 @@ void ConnectWidget::timerEvent(QTimerEvent *) {
   // this is a "wait until cancel" entry
 
   if(vmain == 20) {
+    gpppdata.setWaitCallback(false);
   }
 }
 
@@ -985,6 +1045,7 @@ void ConnectWidget::pause() {
 
 
 void ConnectWidget::cancelbutton() {
+  gpppdata.setWaitCallback(false);
   Modem::modem->stop();
   killTimer(main_timer_ID);
   timeout_timer->stop();
@@ -994,7 +1055,7 @@ void ConnectWidget::cancelbutton() {
   if (termwindow) {
     delete termwindow;
     termwindow = 0L;
-    this->show();
+    show();
   }
 
   messg->setText(i18n("One moment please..."));
@@ -1009,7 +1070,7 @@ void ConnectWidget::cancelbutton() {
   Requester::rq->killPPPDaemon();
   Modem::modem->hangup();
 
-  this->hide();
+  hide();
   messg->setText("");
   p_kppp->quit_b->setFocus();
   p_kppp->show();
@@ -1041,7 +1102,7 @@ void ConnectWidget::script_timed_out() {
     prompt->hide();
 
   prompt->setConsumed();
-  messg->setText(i18n("Script timed out!"));
+  messg->setText(i18n("Script timed out."));
   Modem::modem->hangup();
   emit stopAccounting();
   p_kppp->con_win->stopClock();
@@ -1130,6 +1191,9 @@ void ConnectWidget::if_waiting_slot() {
   p_kppp->statdlg->take_stats(); // start taking ppp statistics
   auto_hostname();
 
+  // signal other applications that we are connected now
+  kapp->dcopClient()->emitDCOPSignal("KpppIface", "connected()", QByteArray());
+
   if(!gpppdata.command_on_connect().isEmpty()) {
     messg->setText(i18n("Running startup command..."));
 
@@ -1147,7 +1211,7 @@ void ConnectWidget::if_waiting_slot() {
   set_con_speed_string();
 
   p_kppp->con_win->setConnectionSpeed(p_kppp->con_speed);
-  this->hide();
+  hide();
   messg->setText("");
 
   // prepare the con_win so as to have the right size for
@@ -1160,7 +1224,7 @@ void ConnectWidget::if_waiting_slot() {
   if (gpppdata.get_dock_into_panel()) {
     DockWidget::dock_widget->show();
     DockWidget::dock_widget->take_stats();
-    this->hide();
+    hide();
   }
   else {
     p_kppp->con_win->show();
@@ -1230,6 +1294,20 @@ bool ConnectWidget::execppp() {
     command += " " + *it;
   }
 
+  // Callback settings
+  if(gpppdata.callbackType() && !gpppdata.waitCallback()) {
+    if(!gpppdata.pppdVersionMin(2, 4, 2)) {
+      command += " +callback";
+      if(gpppdata.callbackType() == CBTYPE_USER)
+        command += " callback " + gpppdata.callbackPhone();
+    } else {
+      command += " callback ";
+      command += gpppdata.callbackType() == CBTYPE_ADMIN ?
+                 QString("0") : gpppdata.callbackPhone();
+    }
+  } else
+    gpppdata.setWaitCallback(false);
+
   // PAP settings
   if(gpppdata.authMethod() == AUTH_PAP) {
     command += " -chap user ";
