@@ -1,8 +1,6 @@
 /*
- *
  *            kPPP: A pppd front end for the KDE project
  *
- * $Id: kpppwidget.cpp,v 1.21 2003/11/29 11:20:49 binner Exp $
  *
  *            Copyright (C) 1997 Bernd Johannes Wuebben
  *                   wuebben@math.cornell.edu
@@ -53,6 +51,7 @@
 #include <khelpmenu.h>
 #include <kpushbutton.h>
 #include <kguiitem.h>
+#include <kstdguiitem.h>
 
 #include <stdlib.h>
 #include <errno.h>
@@ -68,14 +67,18 @@
 #include "pppstats.h"
 #include "pppdata.h"
 #include "general.h"
+#include "modems.h"
 
+// delay disconnection for a second
+#define DISCONNECT_DELAY 1000
 
 extern KPPPWidget *p_kppp;
 
 KPPPWidget::KPPPWidget( QWidget *parent, const char *name )
-  : QWidget(parent, name)
+  : DCOPObject( "KpppIface" ), QWidget(parent, name)
   , acct(0)
   , m_bCmdlAccount (false)
+  , m_bCmdlModem (false)
 {
   tabWindow = 0;
 
@@ -89,7 +92,7 @@ KPPPWidget::KPPPWidget( QWidget *parent, const char *name )
 
   QVBoxLayout *tl = new QVBoxLayout(this, 10, 10);
 
-  QGridLayout *l1 = new QGridLayout(3, 4);
+  l1 = new QGridLayout(4, 4);
   tl->addLayout(l1);
   l1->addColSpacing(0, 10);
   l1->addColSpacing(3, 10);
@@ -106,13 +109,23 @@ KPPPWidget::KPPPWidget( QWidget *parent, const char *name )
 	  SLOT(newdefaultaccount(int)));
   l1->addWidget(connectto_c, 0, 2);
 
+  label7 = new QLabel(i18n("Use &modem: "), this);
+  // l1->addWidget(label7, 1, 1); (done in resetmodems())
+  modem_c = new QComboBox(false, this);
+  label7->setBuddy(connectto_c);
+  m_bModemCShown = false;
+
+  connect(modem_c, SIGNAL(activated(int)),
+	  SLOT(newdefaultmodem(int)));
+  // l1->addWidget(modem_c, 1, 2); (done in resetmodems())
+
   ID_Label = new QLabel(i18n("&Login ID:"), this);
-  l1->addWidget(ID_Label, 1, 1);
+  l1->addWidget(ID_Label, 2, 1);
 
   // the entry line for usernames
   ID_Edit = new QLineEdit(this);
   ID_Label->setBuddy(ID_Edit);
-  l1->addWidget(ID_Edit, 1, 2);
+  l1->addWidget(ID_Edit, 2, 2);
   connect(ID_Edit, SIGNAL(returnPressed()),
 	  this, SLOT(enterPressedInID()));
   QString tmp = i18n("<p>Type in the username that you got from your\n"
@@ -121,18 +134,18 @@ KPPPWidget::KPPPWidget( QWidget *parent, const char *name )
 		     "terminal-based or script-based authentication.\n"
 		     "\n"
 		     "<b>Important</b>: case is important here:\n"
-		     "<i>myusername</i> is not the same as <i>MyUserName</i>!");
+		     "<i>myusername</i> is not the same as <i>MyUserName</i>.");
 
   QWhatsThis::add(ID_Label,tmp);
   QWhatsThis::add(ID_Edit,tmp);
 
   PW_Label = new QLabel(i18n("&Password:"), this);
-  l1->addWidget(PW_Label, 2, 1);
+  l1->addWidget(PW_Label, 3, 1);
 
   PW_Edit= new QLineEdit(this);
   PW_Label->setBuddy(PW_Edit);
   PW_Edit->setEchoMode(QLineEdit::Password);
-  l1->addWidget(PW_Edit, 2, 2);
+  l1->addWidget(PW_Edit, 3, 2);
   connect(PW_Edit, SIGNAL(returnPressed()),
 	  this, SLOT(enterPressedInPW()));
 
@@ -142,7 +155,7 @@ KPPPWidget::KPPPWidget( QWidget *parent, const char *name )
 	     "terminal-based or script-based authentication.\n"
 	     "\n"
 	     "<b>Important</b>: case is important here:\n"
-	     "<i>mypassword</i> is not the same as <i>MyPassword</i>!");
+	     "<i>mypassword</i> is not the same as <i>MyPassword</i>.");
 
   QWhatsThis::add(PW_Label,tmp);
   QWhatsThis::add(PW_Edit,tmp);
@@ -174,12 +187,12 @@ KPPPWidget::KPPPWidget( QWidget *parent, const char *name )
   tl->addLayout(l2);
 
   int minw = 0;
-  quit_b = new KPushButton(KGuiItem(i18n("&Quit"), "exit" ), this);
+  quit_b = new KPushButton(KStdGuiItem::quit(), this);
   connect( quit_b, SIGNAL(clicked()), SLOT(quitbutton()));
   if(quit_b->sizeHint().width() > minw)
       minw = quit_b->sizeHint().width();
 
-  setup_b = new KPushButton(KGuiItem(i18n("Co&nfigure...")), this);
+  setup_b = new KPushButton(KGuiItem(i18n("Co&nfigure..."), "configure"), this);
   connect( setup_b, SIGNAL(clicked()), SLOT(expandbutton()));
   if(setup_b->sizeHint().width() > minw)
       minw = setup_b->sizeHint().width();
@@ -187,12 +200,11 @@ KPPPWidget::KPPPWidget( QWidget *parent, const char *name )
   if(gpppdata.access() != KConfig::ReadWrite)
     setup_b->setEnabled(false);
 
-  help_b = new KPushButton(i18n("&Help"), this);
+  help_b = new KPushButton(KStdGuiItem::help(), this);
   connect( help_b, SIGNAL(clicked()), SLOT(helpbutton()));
 
   KHelpMenu *helpMenu = new KHelpMenu(this, KGlobal::instance()->aboutData(), true);
   help_b->setPopup((QPopupMenu*)helpMenu->menu());
-  help_b->setGuiItem (KGuiItem(i18n("&Help"), "help" ) );
 
   if(help_b->sizeHint().width() > minw)
       minw = help_b->sizeHint().width();
@@ -241,6 +253,7 @@ KPPPWidget::KPPPWidget( QWidget *parent, const char *name )
   // load up the accounts combo box
 
   resetaccounts();
+  resetmodems();
   con = new ConnectWidget(0, "con", stats);
   KWin::setIcons(con->winId(), kapp->icon(), kapp->miniIcon() );
   connect(this, SIGNAL(begin_connect()),con, SLOT(preinit()));
@@ -275,6 +288,7 @@ KPPPWidget::KPPPWidget( QWidget *parent, const char *name )
   KCmdLineArgs *args = KCmdLineArgs::parsedArgs();
 
   m_strCmdlAccount = args->getOption("c");
+  m_strCmdlModem = args->getOption("m");
   m_bQuitOnDisconnect = args->isSet("q");
 
   if(!m_strCmdlAccount.isEmpty()) {
@@ -282,6 +296,21 @@ KPPPWidget::KPPPWidget( QWidget *parent, const char *name )
     kdDebug(5002) << "cmdl_account: " << m_bCmdlAccount << endl;
   }
 
+  if(!m_strCmdlModem.isEmpty()) {
+    m_bCmdlModem = true;
+    kdDebug(5002) << "cmdl_modem: " << m_bCmdlModem << endl;
+  }
+
+  if(m_bCmdlModem){
+    bool result = gpppdata.setModem(m_strCmdlModem);
+    if (!result){
+      QString string;
+      string = i18n("No such Modem:\n%1\nFalling back to default").arg(m_strCmdlModem);
+      KMessageBox::error(this, string);
+      m_bCmdlModem = false;
+    }
+  }
+
   if(m_bCmdlAccount){
     bool result = gpppdata.setAccount(m_strCmdlAccount);
     if (!result){
@@ -289,7 +318,7 @@ KPPPWidget::KPPPWidget( QWidget *parent, const char *name )
       string = i18n("No such Account:\n%1").arg(m_strCmdlAccount);
       KMessageBox::error(this, string);
       m_bCmdlAccount = false;
-      this->show();
+      show();
     } else {
       beginConnect();
     }
@@ -303,6 +332,17 @@ KPPPWidget::KPPPWidget( QWidget *parent, const char *name )
   if(!m_bCmdlAccount)
     showNews();
 #endif
+
+  // attach to the DCOP server, if possible
+  if (!kapp->dcopClient()->attach())
+    kdDebug(5002) << "Error: Could not connect to the DCOP server" << endl;
+  else
+    kapp->dcopClient()->registerAs(kapp->name(), true);
+
+  // this timer will delay the actual disconnection DISCONNECTION_DELAY ms
+  // to give applications time to shutdown, logout, whatever..
+  disconnectTimer = new QTimer(this);
+  connect(disconnectTimer, SIGNAL(timeout()), this, SLOT(delayedDisconnect()));
 }
 
 KPPPWidget::~KPPPWidget()
@@ -358,8 +398,10 @@ void KPPPWidget::prepareSetupDialog() {
     connect(accounts, SIGNAL(resetVolume(const QString &)),
 	    this, SLOT(resetVolume(const QString &)));
 
-    modem1 = new ModemWidget( tabWindow->addPage( i18n("&Device"), i18n("Serial Device") ) );
-    modem2 = new ModemWidget2( tabWindow->addPage( i18n("&Modem"), i18n("Modem Settings") ) );
+    modems = new ModemsWidget(tabWindow->addPage( i18n("&Modems"), i18n("Modems Setup") ) );
+    connect(modems, SIGNAL(resetmodems()),
+	    this, SLOT(resetmodems()));
+
     graph = new GraphSetup( tabWindow->addPage( i18n("&Graph"), i18n("Throughput Graph" ) ) );
     general = new GeneralWidget( tabWindow->addPage( i18n("M&isc"), i18n("Miscellaneous Settings") ) );
   }
@@ -403,18 +445,15 @@ void KPPPWidget::setup() {
 void KPPPWidget::resetaccounts() {
   connectto_c->clear();
 
-  int count = gpppdata.count();
+  int count = gpppdata.accountCount();
 
   // enable/disable controls
   connectto_c->setEnabled(count > 0);
-  connect_b->setEnabled(count > 0);
-  log->setEnabled(count > 0);
-  ID_Edit->setEnabled(count > 0);
-  PW_Edit->setEnabled(count > 0);
+  setButtons();
 
   //load the accounts
   for(int i=0; i < count; i++) {
-    gpppdata.setAccountbyIndex(i);
+    gpppdata.setAccountByIndex(i);
      connectto_c->insertItem(gpppdata.accname());
   }
 
@@ -423,7 +462,7 @@ void KPPPWidget::resetaccounts() {
     for(int i=0; i < count; i++)
        if(gpppdata.defaultAccount() == connectto_c->text(i)) {
  	connectto_c->setCurrentItem(i);
-	gpppdata.setAccountbyIndex(i);
+	gpppdata.setAccountByIndex(i);
 
 	ID_Edit->setText(gpppdata.storedUsername());
 	PW_Edit->setText(gpppdata.storedPassword());
@@ -447,6 +486,65 @@ void KPPPWidget::resetaccounts() {
       ID_Edit->setFocus();
   else if (PW_Edit->text().isEmpty())
       PW_Edit->setFocus();
+
+}
+
+void KPPPWidget::resetmodems() {
+  modem_c->clear();
+
+  int count = gpppdata.modemCount();
+
+  // enable/disable controls
+  modem_c->setEnabled(count > 0);
+  setButtons();
+
+  //load the modems
+  for(int i=0; i < count; i++) {
+    gpppdata.setModemByIndex(i);
+    modem_c->insertItem(gpppdata.modname());
+  }
+
+  if (count > 1 && !m_bModemCShown) {
+      l1->addWidget(label7, 1, 1);
+      l1->addWidget(modem_c, 1, 2);
+      m_bModemCShown = true;
+  } else if (count <= 1 && m_bModemCShown){
+      l1->remove(label7);
+      l1->remove(modem_c);
+      m_bModemCShown = false;
+  }
+  label7->setShown(m_bModemCShown);
+  modem_c->setShown(m_bModemCShown);     
+  layout()->invalidate();
+  setFixedSize(sizeHint());
+
+  //set the default modem
+  if(!gpppdata.defaultModem().isEmpty()) {
+    for(int i=0; i < count; i++)
+       if(gpppdata.defaultModem() == modem_c->text(i)) {
+ 	modem_c->setCurrentItem(i);
+	gpppdata.setModemByIndex(i);
+
+    }
+  }
+  else
+    if(count > 0) {
+       gpppdata.setDefaultModem(modem_c->text(0));
+        gpppdata.save();
+    }
+}
+
+void KPPPWidget::setButtons()
+{
+	int acccount = gpppdata.accountCount();
+	int modcount = gpppdata.modemCount();
+
+  // enable/disable controls
+  connect_b->setEnabled(acccount > 0 && modcount > 0);
+  log->setEnabled(acccount > 0 && modcount > 0);
+  ID_Edit->setEnabled(acccount > 0 && modcount > 0);
+  PW_Edit->setEnabled(acccount > 0 && modcount > 0);
+
 }
 
 
@@ -457,10 +555,9 @@ void KPPPWidget::interruptConnection() {
 
   // disconnect if online
   if (gpppdata.pppdRunning())
-    emit disconnect();
+    emit disconnect();		/* FIXME: are we emitting a slot here!!!??? */
 }
 
-
 void KPPPWidget::sigPPPDDied() {
   kdDebug(5002) << "Received a SIGUSR1" << endl;
 
@@ -480,6 +577,11 @@ void KPPPWidget::sigPPPDDied() {
 
       gpppdata.setpppdRunning(false);
 
+      // stop the disconnect timer (just in case)
+      disconnectTimer->stop();
+      // signal other applications that we are disconnected now
+      kapp->dcopClient()->emitDCOPSignal("KpppIface", "disconnected()", QByteArray());
+
       kdDebug(5002) << "Executing command on disconnect since pppd has died." << endl;
       QApplication::flushX();
       execute_command(gpppdata.command_on_disconnect());
@@ -496,7 +598,10 @@ void KPPPWidget::sigPPPDDied() {
       Modem::modem->unlockdevice();
       con->pppdDied();
 
-      if(!gpppdata.automatic_redial()) {
+      Requester::rq->pppdExitStatus();
+      gpppdata.setWaitCallback(gpppdata.callbackType() && Requester::rq->lastStatus == E_CBCP_WAIT);
+
+      if(!gpppdata.automatic_redial() && !gpppdata.waitCallback()) {
 	quit_b->setFocus();
 	show();
 	con_win->stopClock();
@@ -509,7 +614,7 @@ void KPPPWidget::sigPPPDDied() {
 	QString msg;
 	if (gpppdata.pppdError() == E_IF_TIMEOUT)
 	    msg = i18n("Timeout expired while waiting for the PPP interface "
-                       "to come up!");
+                       "to come up.");
 	else {
 	    msg = i18n("<p>The pppd daemon died unexpectedly!</p>");
 	    Requester::rq->pppdExitStatus();
@@ -517,15 +622,18 @@ void KPPPWidget::sigPPPDDied() {
 		msg += i18n("<p>Exit status: %1").arg(Requester::rq->lastStatus);
 		msg += i18n("</p><p>See 'man pppd' for an explanation of the error "
 			    "codes or take a look at the kppp FAQ on "
-			    "  <a href=http://devel-home.kde.org/~kppp/index.html>"
-			    "http://devel-home.kde.org/~kppp/index.html</a></p>");
+			    "  <a href=\"%1\">%1</a></p>")
+                       .arg("http://developer.kde.org/~kppp/index.html");
 	    }
 	}
 
-	if(KMessageBox::warningYesNo(0, msg, i18n("Error"), i18n("&OK"), i18n("&Details...")) == KMessageBox::No)
+	if(KMessageBox::warningYesNo(0, msg, i18n("Error"), KStdGuiItem::ok(), i18n("&Details...")) == KMessageBox::No)
 	  PPPL_ShowLog();
       } else { /* reconnect on disconnect */
-	kdDebug(5002) << "Trying to reconnect... " << endl;
+        if(gpppdata.waitCallback())
+          kdDebug(5002) << "Waiting for callback... " << endl;
+        else
+          kdDebug(5002) << "Trying to reconnect... " << endl;
 
         if(gpppdata.authMethod() == AUTH_PAP ||
 	   gpppdata.authMethod() == AUTH_CHAP ||
@@ -538,7 +646,7 @@ void KPPPWidget::sigPPPDDied() {
 	con_win->stopClock();
 	stopAccounting();
 	gpppdata.setpppdRunning(false);
-	// not in a signal handler !!!	KNotifyClient::beep();
+	// not in a signal handler !!!  KNotifyClient::beep();
 	emit cmdl_start();
     }
   }
@@ -569,6 +677,12 @@ void KPPPWidget::newdefaultaccount(int i) {
   PW_Edit->setText(gpppdata.storedPassword());
 }
 
+void KPPPWidget::newdefaultmodem(int i) {
+  gpppdata.setDefaultModem(modem_c->text(i));
+  gpppdata.save();
+}
+
+
 
 void KPPPWidget::expandbutton() {
   setup();
@@ -607,7 +721,14 @@ void KPPPWidget::beginConnect() {
   }
 #endif
 
-  QFileInfo info2(gpppdata.modemDevice());
+  KCmdLineArgs *args = KCmdLineArgs::parsedArgs();
+  QString device = "";
+  if (args->isSet("dev"))
+    device = args->getOption("dev");
+  else
+    device = gpppdata.modemDevice();
+
+  QFileInfo info2(device);
 
   if(!info2.exists()){
     QString string;
@@ -615,7 +736,7 @@ void KPPPWidget::beginConnect() {
 		   "your modem device properly "
 		   "and/or adjust the location of the modem device on "
 		   "the modem tab of "
-		   "the setup dialog.").arg(gpppdata.modemDevice());
+		   "the setup dialog.").arg(device);
     KMessageBox::error(this, string);
     return;
   }
@@ -630,7 +751,7 @@ void KPPPWidget::beginConnect() {
 			   i18n(
                            "You have selected the authentication "
 			   "method PAP or CHAP. This requires that you "
-			   "supply a username and a password!"));
+			   "supply a username and a password."));
       return;
     } else {
       if(!Requester::rq->setSecret(gpppdata.authMethod(),
@@ -646,16 +767,16 @@ void KPPPWidget::beginConnect() {
   }
 
   if (gpppdata.phonenumber().isEmpty()) {
-    QString s = i18n("You must specify a telephone number!");
+    QString s = i18n("You must specify a telephone number.");
     KMessageBox::error(this, s);
     return;
   }
 
-  this->hide();
+  hide();
 
   QString tit = i18n("Connecting to: %1").arg(gpppdata.accname());
   con->setCaption(tit);
-
+  con->enableButtons();
   con->show();
 
   bool show_debug = gpppdata.get_show_log_window();
@@ -671,12 +792,17 @@ void KPPPWidget::beginConnect() {
   emit begin_connect();
 }
 
-
 void KPPPWidget::disconnect() {
+  if (disconnectTimer->isActive()) return; // you had already pressed disconnect before
+
+  // signal other applications that we are about to go offline now
+  kapp->dcopClient()->emitDCOPSignal("KpppIface", "aboutToDisconnect()", QByteArray());
+  con_win->hide();
+  con->show();
+  con->disableButtons(); // will reenable them later in delayedDisconnect()
+  con->setCaption(i18n("Disconnecting..."));
+
   if (!gpppdata.command_before_disconnect().isEmpty()) {
-    con_win->hide();
-    con->show();
-    con->setCaption(i18n("Disconnecting..."));
     con->setMsg(i18n("Executing command before disconnection."));
 
     kapp->processEvents();
@@ -689,15 +815,27 @@ void KPPPWidget::disconnect() {
       i = waitpid(id, &status, WNOHANG);
       usleep(500000);
     } while (i == 0 && errno == 0);
-
-    con->hide();
   }
 
-  kapp->processEvents();
+  con->setMsg(i18n("Announcing disconnection."));
+
+  // this is no longer necessary since I'm delaying disconnection usign a QTimer
+  //  kapp->processEvents();
+
+  // set the timer to call delayedDisconnect() in DISCONNECT_DELAY ms
+  disconnectTimer->start(DISCONNECT_DELAY, true);
+}
+
+void KPPPWidget::delayedDisconnect() {
+  // this is where the actual disconnection takes place
+  con->hide();
 
   statdlg->stop_stats();
   Requester::rq->killPPPDaemon();
 
+  // signal other applications that we are disconnected now
+  kapp->dcopClient()->emitDCOPSignal("KpppIface", "disconnected()", QByteArray());
+
   QApplication::flushX();
   execute_command(gpppdata.command_on_disconnect());
 
@@ -717,8 +855,8 @@ void KPPPWidget::disconnect() {
   if(m_bQuitOnDisconnect)
     kapp->exit(0);
   else {
-    this->quit_b->setFocus();
-    this->show();
+    quit_b->setFocus();
+    show();
   }
 }
 
@@ -735,6 +873,13 @@ void KPPPWidget::quitbutton() {
 			    i18n("Quit kPPP?"));
     if(ok == KMessageBox::Yes) {
       Requester::rq->killPPPDaemon();
+
+      // stop the disconnect delay timer
+      disconnectTimer->stop();
+
+      // signal other applications that we are disconnected now
+      kapp->dcopClient()->emitDCOPSignal("KpppIface", "disconnected()", QByteArray());
+
       QApplication::flushX();
       execute_command(gpppdata.command_on_disconnect());
       removedns();
@@ -758,7 +903,7 @@ void KPPPWidget::startAccounting() {
   // volume accounting
   stats->totalbytes = 0;
 
-  kdDebug() << "AcctEnabled: " << gpppdata.AcctEnabled() << endl;
+  kdDebug(5002) << "AcctEnabled: " << gpppdata.AcctEnabled() << endl;
 
   // load the ruleset
   if(!gpppdata.AcctEnabled())
@@ -776,7 +921,7 @@ void KPPPWidget::startAccounting() {
 
   if(!acct->loadRuleSet(gpppdata.accountingFile())) {
     QString s= i18n("Can not load the accounting "
-    		    "ruleset \"%1\"!").arg(gpppdata.accountingFile());
+    		    "ruleset \"%1\".").arg(gpppdata.accountingFile());
 
     // starting the messagebox with a timer will prevent us
     // from blocking the calling function ConnectWidget::timerEvent
@@ -895,7 +1040,7 @@ void KPPPWidget::showNews() {
 
     KButtonBox *bbox = new KButtonBox(&dlg);
     bbox->addStretch(1);
-    QPushButton *ok = bbox->addButton(i18n("OK"));
+    QPushButton *ok = bbox->addButton(KStdGuiItem::ok());
     ok->setDefault(true);
     dlg.connect(ok, SIGNAL(clicked()),
 		&dlg, SLOT(accept()));
