@@ -17,7 +17,23 @@
 #include "netscanner.h"
 #include "ipaddress.h"
 #include "lisadefines.h"
-#include "getdebug.h"
+#include <iostream>
+
+#ifdef LISA_DEBUG
+#undef LISA_DEBUG
+#endif
+#define LISA_DEBUG 0
+
+#ifdef dcerr
+#undef dcerr
+#endif
+
+#ifdef mdcerr
+#undef mdcerr
+#endif
+
+#define dcerr if (LISA_DEBUG==1) std::cerr<<"NetScanner::"
+#define mdcerr if (LISA_DEBUG==1) std::cerr<<procId<<" NetScanner::"
 
 #include <stdio.h>
 #include <sys/socket.h>
@@ -36,7 +52,9 @@
 #include <netinet/ip.h>
 #include <netinet/ip_icmp.h>
 
-#define getDebug() getDebug()<<procId
+#ifndef INADDR_NONE
+#define INADDR_NONE -1
+#endif
 
 struct ICMPEchoRequest
 {
@@ -93,20 +111,21 @@ NetScanner::NetScanner(int& rawSocketFD, int strictMode)
 ,m_useNmblookup(0)
 ,hostList(0)
 ,tmpIPRange("")
-{};
+{}
 
 NetScanner::~NetScanner()
 {
+//   std::cerr<<"----------- Netscanner dtor "<<std::endl;
    delete hostList;
    ::close(m_rawSocketFD);
-};
+}
 
 void addMissingSemicolon(MyString& text)
 {
    if (text.isEmpty()) return;
    if (text[text.length()-1]!=';')
       text+=';';
-};
+}
 
 void NetScanner::configure(Config& config)
 {
@@ -115,7 +134,7 @@ void NetScanner::configure(Config& config)
    {
       ipRangeStr=stripWhiteSpace(config.getEntry("PingAddresses",""));
       addMissingSemicolon(ipRangeStr);
-   };
+   }
    MyString pingNames=stripWhiteSpace(config.getEntry("PingNames",""));
    addMissingSemicolon(pingNames);
    MyString nextName;
@@ -124,7 +143,7 @@ void NetScanner::configure(Config& config)
    while (semicolonPos!=-1)
    {
       nextName=pingNames.left(semicolonPos);
-      getDebug()<<"NetScanner::configure(): looking up -"<<nextName<<"-"<<endl;
+      mdcerr<<"configure(): looking up -"<<nextName<<"-"<<std::endl;
       //now the name lookup
       in_addr server_addr;
       hostent *hp=gethostbyname(nextName.data());
@@ -132,16 +151,16 @@ void NetScanner::configure(Config& config)
       {
          if ((m_strictMode) && (hostsAdded>=STRICTMODEMAXHOSTS))
             break;
-         memcpy(&server_addr, hp->h_addr, hp->h_length);
+         memcpy(&server_addr, hp->h_addr, sizeof(server_addr));
          char *ip=inet_ntoa(server_addr);
-         getDebug()<<"NetScanner::configure(): looking up "<<nextName<<" gives -"<<ip<<"-"<<endl;
+         mdcerr<<"configure(): looking up "<<nextName<<" gives -"<<ip<<"-"<<std::endl;
          ipRangeStr=ipRangeStr+ip+';';
          hostsAdded++;
       }
 
       pingNames=pingNames.mid(semicolonPos+1);
       semicolonPos=pingNames.find(';');
-   };
+   }
    if ((!ipRangeStr.isEmpty()) && (ipRangeStr[0]==';'))
       ipRangeStr=ipRangeStr.mid(1);
    m_deliverUnnamedHosts=config.getEntry("DeliverUnnamedHosts",0);
@@ -156,45 +175,45 @@ void NetScanner::configure(Config& config)
    // if the microseconds are more than 1.000.000
    if (m_firstWait>99) m_firstWait=99;
    if (m_secondWait>99) m_secondWait=99;
-   getDebug()<<"NetScanner::configure(): "<<ipRangeStr<<endl;
-};
+   mdcerr<<"configure(): "<<ipRangeStr<<std::endl;
+}
 
 struct in_addr NetScanner::getIPfromArray(unsigned int index)
 {
-   //getDebug()<<endl<<"*** start ***"<<endl;
+   //mdcerr<<std::endl<<"*** start ***"<<std::endl;
    unsigned int tmpIndex(0),indexLeft(index);
    resetIPRange();
    MyString tmp(getNextIPRange());
-//   getDebug()<<"NetScanner::getIPFromArray: -"<<tmp<<"-"<<endl;
-   while (tmp!="")
+//   mdcerr<<"NetScanner::getIPFromArray: -"<<tmp<<"-"<<std::endl;
+   while (!tmp.isEmpty())
    {
       if (tmp.contains('/'))
       {
-         //getDebug()<<"net/mask combination detected"<<endl;
+         //mdcerr<<"net/mask combination detected"<<std::endl;
          MyString netStr(tmp.left(tmp.find("/")));
          MyString maskStr(tmp.mid(tmp.find("/")+1));
-         unsigned int net(IPAddress(netStr).asInt());
          unsigned int mask(IPAddress(maskStr).asInt());
+         unsigned int net(IPAddress(netStr).asInt()&mask);
          if ((~mask)<indexLeft)
          {
             indexLeft=indexLeft-(~mask+1);
             tmpIndex+=(~mask)+1;
-            //getDebug()<<"i: "<<tmpIndex<<" left: "<<indexLeft<<endl;
+            //mdcerr<<"i: "<<tmpIndex<<" left: "<<indexLeft<<std::endl;
          }
          else
          {
             net+=indexLeft;
             return IPAddress(net).asStruct();
             //return string2Struct(ipInt2String(net));
-         };
+         }
       }
       else if (tmp.contains('-')==1)
       {
-         //getDebug()<<"single range detected"<<endl;
+         //mdcerr<<"single range detected"<<std::endl;
          MyString fromIPStr(tmp.left(tmp.find("-")));
          MyString toIPStr(tmp.mid(tmp.find("-")+1));
-         //getDebug()<<"fromIPStr: "<<fromIPStr<<endl;
-         //getDebug()<<"toIPStr: "<<toIPStr<<endl;
+         //mdcerr<<"fromIPStr: "<<fromIPStr<<std::endl;
+         //mdcerr<<"toIPStr: "<<toIPStr<<std::endl;
          unsigned int fromIP(IPAddress(fromIPStr).asInt());
          unsigned int toIP(IPAddress(toIPStr).asInt());
          //unsigned int fromIP(ipString2Int(fromIPStr)), toIP(ipString2Int(toIPStr));
@@ -203,7 +222,7 @@ struct in_addr NetScanner::getIPfromArray(unsigned int index)
          {
             tmpIndex+=1+toIP-fromIP;
             indexLeft=indexLeft-(1+toIP-fromIP);
-            //getDebug()<<"i: "<<tmpIndex<<" left: "<<indexLeft<<endl;
+            //mdcerr<<"i: "<<tmpIndex<<" left: "<<indexLeft<<std::endl;
          }
          //index in diesem bereich
          else
@@ -211,12 +230,12 @@ struct in_addr NetScanner::getIPfromArray(unsigned int index)
             fromIP+=indexLeft;
             return IPAddress(fromIP).asStruct();
             //return string2Struct(ipInt2String(fromIP));
-         };
+         }
          
       }
       else if (tmp.contains('-')==4)
       {
-         //getDebug()<<"multiple range detected"<<endl;
+         //mdcerr<<"multiple range detected"<<std::endl;
          int cp(tmp.find('-'));
          int from1(atoi(tmp.left(cp).data()));
          tmp=tmp.mid(cp+1);
@@ -252,7 +271,7 @@ struct in_addr NetScanner::getIPfromArray(unsigned int index)
          {
             tmpIndex+=count;
             indexLeft-=count;
-            //getDebug()<<"i: "<<tmpIndex<<" left: "<<indexLeft<<endl;
+            //mdcerr<<"i: "<<tmpIndex<<" left: "<<indexLeft<<std::endl;
          }
          else
          {
@@ -267,33 +286,33 @@ struct in_addr NetScanner::getIPfromArray(unsigned int index)
                         };
                         tmpIndex++;
                         indexLeft--;
-                        //getDebug()<<"i: "<<tmpIndex<<" left: "<<indexLeft<<endl;
-                     };
-         };
+                        //mdcerr<<"i: "<<tmpIndex<<" left:"<<indexLeft<<std::endl;
+                     }
+         }
       }
       //single IP address
       else if (tmp.contains('.')==3)
       {
-         //getDebug()<<"single IP address detected"<<endl;
+         //mdcerr<<"single IP address detected"<<std::endl;
          //if (tmpIndex==index) return string2Struct(tmp);
          if (tmpIndex==index) return IPAddress(tmp).asStruct();
          else
          {
             tmpIndex++;
             indexLeft--;
-            //getDebug()<<"i: "<<tmpIndex<<" left: "<<indexLeft<<endl;
-         };
-      };
-      //getDebug()<<"nextIPRange: *"<<tmp<<"*"<<endl;
+            //mdcerr<<"i: "<<tmpIndex<<" left: "<<indexLeft<<std::endl;
+         }
+      }
+      //mdcerr<<"nextIPRange: *"<<tmp<<"*"<<std::endl;
       tmp=getNextIPRange();
-   };
+   }
    return IPAddress("0.0.0.0").asStruct();
-};
+}
 
 void NetScanner::resetIPRange()
 {
    tmpIPRange=ipRangeStr;
-};
+}
 
 MyString NetScanner::getNextIPRange()
 {
@@ -302,7 +321,7 @@ MyString NetScanner::getNextIPRange()
    MyString tmp(tmpIPRange.left(cp));
    tmpIPRange=tmpIPRange.mid(cp+1);
    return tmp;
-};
+}
 
 char* NetScanner::ip2Name(struct in_addr ip)
 {
@@ -310,22 +329,22 @@ char* NetScanner::ip2Name(struct in_addr ip)
    // Set the hostname of node
    if ( ( hostname = gethostbyaddr( (char *) &ip.s_addr, 4, AF_INET  ) ) == 0 )
    {
-      getDebug() << "TCPNode::TCPNode->gethostbyname* error" << endl;
+      mdcerr << "ip2Name gethostbyname* error" << std::endl;
       return inet_ntoa( ip );
    }
-   getDebug()<<"NetScanner::ip2name -"<<hostname->h_name<<endl;
+   mdcerr<<"ip2name -"<<hostname->h_name<<std::endl;
    return hostname->h_name;
-};
+}
 
-void NetScanner::nmblookupScan(SimpleList<Node>* newList)
+void NetScanner::nmblookupScan(std::list<Node>* newList)
 {
-   getDebug()<<"NetScanner::nmblookupScan()"<<endl;
+   mdcerr<<"nmblookupScan()"<<std::endl;
    //newList->clear();
    FILE * nmblookupFile=popen("nmblookup \"*\"","r");
    //no success
    if (nmblookupFile==0)
    {
-      getDebug()<<"NetScanner::nmblookupScan(): could not start nmblookup"<<endl;
+      mdcerr<<"nmblookupScan(): could not start nmblookup"<<std::endl;
       return;
    };
    MyString dummy("");
@@ -369,13 +388,16 @@ void NetScanner::nmblookupScan(SimpleList<Node>* newList)
             {
                memcpy(newBuf,receiveBuffer,bufferSize);
                delete [] receiveBuffer;
-            };
+            }
             memcpy(newBuf+bufferSize,tmpBuf,bytesRead);
             receiveBuffer=newBuf;
             bufferSize+=bytesRead;
-         };
-      };
+         }
+      }
    } while (!done);
+
+   // Warning: The return value of pclose may be incorrect due to the
+   // SIGCHLD handler that is installed. Ignore it!
    pclose(nmblookupFile);
 
    delete [] tmpBuf;
@@ -387,7 +409,7 @@ void NetScanner::nmblookupScan(SimpleList<Node>* newList)
    if (receiveBuffer[bufferSize-1]=='\0')
       receiveBuffer[bufferSize-1]='\0';
 
-   //cerr<<receiveBuffer<<endl;
+   //std::cerr<<receiveBuffer<<std::endl;
 
    tmpBuf=receiveBuffer;
    int bytesLeft=bufferSize;
@@ -395,7 +417,7 @@ void NetScanner::nmblookupScan(SimpleList<Node>* newList)
    int bufferState=0;
    while (bytesLeft>0)
    {
-      //getDebug()<<"bytesLeft: "<<bytesLeft<<" received: "<<bufferSize<<endl;
+      //mdcerr<<"bytesLeft: "<<bytesLeft<<" received: "<<bufferSize<<std::endl;
       //since we added a terminating \0 we can be sure here
       char *endOfLine=(char*)memchr(tmpBuf,'\n',bytesLeft);
       //point to the last character
@@ -416,14 +438,14 @@ void NetScanner::nmblookupScan(SimpleList<Node>* newList)
       {
          if (isdigit(tmpBuf[0]))
             bufferState=1;
-      };
+      }
       //yes, no else !
       if (bufferState==1)
       {
          char tmpIP[1024];
-         //cerr<<"tmpBuf: -"<<tmpBuf<<"-"<<endl;
+         //std::cerr<<"tmpBuf: -"<<tmpBuf<<"-"<<std::endl;
          sscanf(tmpBuf,"%s *<00>\n",tmpIP);
-         getDebug()<<"NetScanner::nmblookupScan: tmpIP: -"<<tmpIP<<"-"<<endl;
+         mdcerr<<"nmblookupScan: tmpIP: -"<<tmpIP<<"-"<<std::endl;
          struct sockaddr_in addr;
          //if (inet_aton(tmpIP,&addr.sin_addr))
          if ((addr.sin_addr.s_addr = inet_addr(tmpIP)) != INADDR_NONE)
@@ -431,34 +453,34 @@ void NetScanner::nmblookupScan(SimpleList<Node>* newList)
             {
                if (validator.isValid(addr.sin_addr.s_addr))
                {
-                  getDebug()<<"NetScanner::nmblookupScan: adding "<<inet_ntoa(addr.sin_addr)<<endl;
-                  newList->append(Node(dummy,addr.sin_addr.s_addr));
-               };
-            };
-      };
+                  mdcerr<<"nmblookupScan: adding "<<inet_ntoa(addr.sin_addr)<<std::endl;
+                  newList->push_back(Node(dummy,addr.sin_addr.s_addr));
+               }
+            }
+      }
       tmpBuf=endOfLine+1;
    };
-   getDebug()<<"NetScanner::nmblookupScan: finished"<<endl;
+   mdcerr<<"nmblookupScan: finished"<<std::endl;
    delete [] receiveBuffer;
-};
+}
 
-void NetScanner::pingScan(SimpleList<Node>* newList)   //the ping-version
+void NetScanner::pingScan(std::list<Node>* newList)   //the ping-version
 {
-   getDebug()<<"NetScanner::pingScan()"<<endl;
+   mdcerr<<"pingScan()"<<std::endl;
    //newList->clear();
    MyString dummy("");
-   getDebug()<<"NetScanner::pingScan: m_maxPings: "<<m_maxPings<<endl;
+   mdcerr<<"pingScan: m_maxPings: "<<m_maxPings<<std::endl;
 
 /*   int bufferSize(60*1024);
-   getDebug()<<"NetScanner::pingScan: regaining root privileges for setting up the icmp socket"<<endl;
+   mdcerr<<"NetScanner::pingScan: regaining root privileges for setting up the icmp socket"<<std::endl;
    seteuid(0);
    int sockFD=socket(AF_INET,SOCK_RAW,IPPROTO_ICMP);
-   getDebug()<<"NetScanner::pingScan: setsockopt returns "<<setsockopt(sockFD,SOL_SOCKET,SO_RCVBUF,&bufferSize,sizeof(bufferSize))<<endl;
+   mdcerr<<"NetScanner::pingScan: setsockopt returns "<<setsockopt(sockFD,SOL_SOCKET,SO_RCVBUF,&bufferSize,sizeof(bufferSize))<<std::endl;
    //make the socket non-blocking
    //long int options = O_NONBLOCK | ::fcntl(sockFD, F_GETFL);
-   //getDebug()<<"NetScanner::pingScan:: made it non blocking: "<<::fcntl( sockFD, F_SETFL, options )<<endl;
+   //mdcerr<<"NetScanner::pingScan:: made it non blocking: "<<::fcntl(sockFD, F_SETFL, options )<<std::endl;
    seteuid(getuid());
-   getDebug()<<"NetScanner::pingScan: dropped root privileges again"<<endl;*/
+   mdcerr<<"NetScanner::pingScan: dropped root privileges again"<<std::endl;*/
 
    pid_t pid=getpid();
    ICMPEchoRequest echo;
@@ -487,7 +509,7 @@ void NetScanner::pingScan(SimpleList<Node>* newList)   //the ping-version
       tv1.tv_usec = 0;
       FD_ZERO(&clearSet);
       FD_SET(m_rawSocketFD,&clearSet);
-   };
+   }
    //now the buffer should be empty
    
    //wait a moment for answers
@@ -495,10 +517,11 @@ void NetScanner::pingScan(SimpleList<Node>* newList)   //the ping-version
    tv1.tv_usec = m_firstWait*10*1000;//0.5 sec
 
    int loopCount(2);
-   if (m_secondWait<0) loopCount=1;
+   if (m_secondWait<0)
+      loopCount=1;
    for (int repeatOnce=0; repeatOnce<loopCount; repeatOnce++)
    {
-      getDebug()<<"******************** starting loop *****************"<<endl;
+      mdcerr<<"******************** starting loop *****************"<<std::endl;
       unsigned int current(0);
       int finished(0);
       while (!finished)
@@ -510,11 +533,11 @@ void NetScanner::pingScan(SimpleList<Node>* newList)   //the ping-version
             {
                tmpIP=getIPfromArray(current);
                current++;
-               getDebug()<<"NetScanner::pingScan(): trying "<<inet_ntoa(tmpIP)<<endl;
+//               mdcerr<<"pingScan(): trying "<<inet_ntoa(tmpIP)<<std::endl;
                if (hostAlreadyInList(tmpIP.s_addr,newList))
-                  getDebug()<<"already in list :-)"<<endl;
+                  mdcerr<<"already in list :-)"<<std::endl;
                if (!validator.isValid(tmpIP.s_addr))
-                  getDebug()<<"NetScanner::pingScan(): invalid IP :-("<<endl;
+                  mdcerr<<"pingScan(): invalid IP :-("<<std::endl;
                //ping only hosts which are allowed to receive the results
                //and which are not in the list
             } while ( (tmpIP.s_addr!=0)
@@ -531,10 +554,10 @@ void NetScanner::pingScan(SimpleList<Node>* newList)   //the ping-version
                toAddr.sin_port = 0;
                int sb=sendto(m_rawSocketFD,(char*)&echo,sizeof(echo),0,(sockaddr*)&toAddr,sizeof(toAddr));
                //int sb=sendto(sockFD,(char*)&echo,sizeof(echo),0,(sockaddr*)&toAddr,sizeof(toAddr));
-               getDebug()<<"NetScanner::pingScan: pinging "<<inet_ntoa(toAddr.sin_addr)<<endl;
+//               mdcerr<<"pingScan: pinging "<<inet_ntoa(toAddr.sin_addr)<<std::endl;
             }
             else break;
-         };
+         }
          select(0,0,0,0,&tv1);
          //now read the answers, hopefully
          struct sockaddr_in fromAddr;
@@ -552,7 +575,7 @@ void NetScanner::pingScan(SimpleList<Node>* newList)   //the ping-version
 			      (sockaddr*)&fromAddr, &length);
             if (received!=-1)
             {
-               getDebug()<<"NetScanner::pingScan: received from "<<inet_ntoa(fromAddr.sin_addr)<<" "<<received<<" b, ";
+//               mdcerr<<"pingScan: received from "<<inet_ntoa(fromAddr.sin_addr)<<" "<<received<<" b, ";
                struct ip *ipFrame=(ip*)&receiveBuf;
                int icmpOffset=(ipFrame->ip_hl)*4;
                icmp *recIcmpFrame=(icmp*)(receiveBuf+icmpOffset);
@@ -565,110 +588,87 @@ void NetScanner::pingScan(SimpleList<Node>* newList)   //the ping-version
                    && (!hostAlreadyInList(fromAddr.sin_addr.s_addr,newList)))
                {
                   //this is an answer to our request :-)
-                  getDebug()<<"NetScanner::pingScan: adding "<<inet_ntoa(fromAddr.sin_addr)<<endl;
-                  newList->append(Node(dummy,fromAddr.sin_addr.s_addr));
-               };
-            };
+//                  mdcerr<<"pingScan: adding "<<inet_ntoa(fromAddr.sin_addr)<<std::endl;
+                  newList->push_back(Node(dummy,fromAddr.sin_addr.s_addr));
+               }
+            }
             tv1.tv_sec=0;
             tv1.tv_usec=0;
             FD_ZERO(&sockFDset);
             FD_SET(m_rawSocketFD,&sockFDset);
             //FD_SET(sockFD,&sockFDset);
-         };
-      };
+         }
+      }
       tv1.tv_sec = 0;
       tv1.tv_usec = m_secondWait*10*1000;//0.5 sec
-   };
-   getDebug()<<"NetScanner::pingScan() ends"<<endl;
-};
+   }
+   mdcerr<<"pingScan() ends"<<std::endl;
+}
 
 void NetScanner::doScan()
 {
-   getDebug()<<" NetScanner::doScan"<<endl;
+   mdcerr<<"doScan"<<std::endl;
    //child
-   SimpleList<Node>* tmpPingList=new SimpleList<Node>;
-   getDebug()<<" NetScanner::doScan: created list"<<endl;
+   std::list<Node>* tmpPingList=new std::list<Node>;
+   mdcerr<<"doScan: created list"<<std::endl;
    if (m_useNmblookup)
       nmblookupScan(tmpPingList);
    pingScan(tmpPingList);
    // get the names from cache or lookup
    completeNames(tmpPingList);
-   getDebug()<<"NetScanner::doScan: completed names"<<endl;
+   mdcerr<<"doScan: completed names"<<std::endl;
    if (m_deliverUnnamedHosts==0)
       removeUnnamedHosts(tmpPingList);
 
-   getDebug()<<"NetScanner::doScan: added hosts"<<endl;
+   mdcerr<<"doScan: added hosts"<<std::endl;
    
    delete hostList;
    hostList=tmpPingList;
-};
+}
 
-int NetScanner::hostAlreadyInList(int ip, SimpleList<Node>* nodes)
+int NetScanner::hostAlreadyInList(int ip, std::list<Node>* nodes)
 {
-   for (Node* node=nodes->first(); node!=0; node=nodes->next())
+   for (std::list<Node>::iterator node = nodes->begin(); node != nodes->end(); node++)
    {
       if (node->ip==ip)
          return 1;
-   };
+   }
    return 0;
-};
+}
 
-void NetScanner::removeUnnamedHosts(SimpleList<Node>* nodes)
+void NetScanner::removeUnnamedHosts(std::list<Node>* nodes)
 {
-   struct in_addr tmpAddr;
-   Node* pre(0);
-   for (Node* node=nodes->first(); node!=0; node=nodes->next())
-   {
-      //in node this is already in NBO
-      tmpAddr.s_addr=node->ip;
-      const char* ip=inet_ntoa(tmpAddr);
-      //this way we only compare the whole string via strcmp()
-      //if at least the first char is already the same
-      if (ip[0]==node->name[0])
-      {
-         //this host has no name
-         if (strcmp(ip,node->name.data())==0)
-         {
-            nodes->remove(node);
-            //set the pointer to the previous item
-            if (pre==0) node=nodes->first();
-            else node=pre;
-         };
-      };
-      pre=node;
-   };
-};
+   nodes->remove_if(is_unnamed_node());
+}
 
-void NetScanner::completeNames(SimpleList<Node>* nodes)
+void NetScanner::completeNames(std::list<Node>* nodes)
 {
    struct sockaddr_in tmpAddr;
    //for every host
-   for (Node* node=nodes->first(); node!=0; node=nodes->next())
+   for (std::list<Node>::iterator node = nodes->begin(); node != nodes->end(); node++)
    {
       tmpAddr.sin_addr.s_addr=node->ip;
-      getDebug()<<"\n NetScanner::completeNames: looking up "<<inet_ntoa(tmpAddr.sin_addr)<<endl;
+      mdcerr<<"completeNames: looking up "<<inet_ntoa(tmpAddr.sin_addr)<<std::endl;
       int done(0);
       //first look wether we have the name already
-      if (hostList!=0) for (Node* oldNode=hostList->first(); oldNode!=0; oldNode=hostList->next())
+      if (hostList!=0) for (std::list<Node>::iterator oldNode=hostList->begin(); oldNode!=hostList->end(); oldNode++)
       {
          if (node->ip==oldNode->ip)
          {
-            getDebug()<<"NetScanner::completeNames: cached: "<<oldNode->name<<" :-)"<<endl;
+            mdcerr<<"completeNames: cached: "<<oldNode->name<<" :-)"<<std::endl;
             node->name=oldNode->name;
             done=1;
             break;
-         };
-      };
+         }
+      }
       //otherwise do a name lookup
       if (!done)
       {
          //IPAddress tmpAddress(node->ip);
-         //getDebug()<<"NetScanner::completeNames: doing actual lookup"<<endl;
+         //mdcerr<<"NetScanner::completeNames: doing actual lookup"<<std::endl;
          node->name=ip2Name(tmpAddr.sin_addr);
-         getDebug()<<"NetScanner::completeNames: resolved: "<<node->name<<endl;
-      };
-   };
-};
-
-#undef getDebug
+         mdcerr<<"completeNames: resolved: "<<node->name<<std::endl;
+      }
+   }
+}
 
